"""

ExpBGFGModel

Estimates the probability that a pixel is foreground, background
in any non-aligned video for a given experiment. Background subtraction
based modeling will further refine this model to a single video. The
model implemented here is the following. 

Let

l = pixel label in {fore,back}.
x = pixel location in image.
y = pixel color (or patch) in current image.
d = distance to closest object detection.

Then

P_exp(l | x, y, d) \propto p_exp(y | l, x) p_exp(d | l) P_exp(l | x)

assuming y \perp d | l, x  and  d \perp x | l

We model p_exp(y | l, x) as a Gaussian mu(l,x), sigma(l,x). This is
done by est_fg_appearance_marginal and est_bg_appearance_marginal.

We approximate p_exp(d | l) by histogramming with a fixed bin size. 

We approximate P_exp(l | x) by histogramming with a fixed bin size.

"""
import getopt
import sys
import os.path
import pickle

import numpy as num
import scipy.ndimage.morphology as morph
import scipy.ndimage.filters as filters
import scipy.io

from params import params
import bg
import movies
import annfiles as annot
import estconncomps as est

if params.interactive:
    import wx

DEBUG = True

if DEBUG:
    import pdb
    import matplotlib.pyplot as plt
    import matplotlib.cm as cm


class ExpBGFGModel:

    def __init__( self, movienames=None, annnames=None, picklefile=None,
                  LoG_hsize=21, 
                  LoG_sigma=5.,
                  LoG_nonmaxsup_sz=5, 
                  LoG_thresh=.5,
                  difftype='darkfglightbg',
                  obj_detection_dist_nbins=100,
                  isback_dilation_hsize=10,
                  fg_min_sigma = 1.,
                  bg_min_sigma = 1.,
                  min_always_bg_mask_frac = 1.,
                  min_frac_cc_covered = .01,
                  min_area_cc_covered = 1,
                  backsub_n_bg_std_thresh = 10.,
                  backsub_n_bg_std_thresh_low = 1.,
                  backsub_do_use_morphology = False,
                  backsub_opening_radius = 0,
                  backsub_closing_radius = 0
                  ):
        """
        ExpBGFGModel(movienames,annnames)

        Initialize the ExpBGFGModel. 

        Inputs: 
        movienames is a list of all movie files to train on
        annnames is a list of the corresponding annotation files

        The pixel-appearance foreground model is initialized using
        init_fg_model.
        The pixel-appearance background model is initialized using
        init_bg_model.
        The foreground detection model is initialized using
        init_obj_detection.

        Optional inputs:
        LoG_hsize: Width, height of the LoG filter
        (radius = (LoG_hsize-1)/2, so this should be an odd number)
        [21]. 
        LoG_sigma: Standard deviation of the LoG filter [5].
        LoG_nonmaxsup_sz: Size of the neighborhood over which nonmaximal
        suppression will be performed for LoG detection [5]. 
        LoG_thresh: Threshold for LoG object detection [.5].
        difftype: Whether the flies are dark on a light background 
        ('darkfglightbg'), light on a dark background ('lightfgdarkbg'),
        or other ('other') ['darkfglightbg']. 
        obj_detection_dist_nbins: Number of log-spaced bins for distance
        to nearest object detection [100]. 
        isback_dilation_hsize: Size of ones matrix for eroding ~isfore
        to compute isback [10]. 
        fg_min_sigma: Minimum standard deviation for the foreground
        pixel appearance [1]. 
        bg_min_sigma: Minimum standard deviation for the background 
        pixel appearance [1]. 
        min_frac_cc_covered: Minimum fraction of a foreground connected component that
        must be covered by the tracked ellipse in order for the component to be added
        as foreground [.01].
        min_area_cc_covered: Minimum number of pixels of a connected component that
        must be covered by the tracked ellipse in order for the component to be added
        as foreground [1].
        backsub_n_bg_std_thresh: High threshold for background subtraction for the 
        sampled frame [10].
        backsub_n_bg_std_thresh_low: Low threshold for background subtraction for the
        sampled frame [1].
        backsub_do_use_morphology: Whether to do morphology after background subtraction
        [False].
        backsub_opening_radius: Radius of opening structural element after background
        subtraction [0].
        backsub_closing_radius: Radius of closing structural element after background
        subtraction [0].         
        """
        
        if picklefile is not None:
            if params.interactive:
                wx.Yield()
                wx.BeginBusyCursor()
            else:
                print 'Loading pickled global bg/fg models'

            try:
                fid = open(picklefile,'r')
                model = pickle.load(fid)
                fid.close()
                model['obj_detection_dist_centers'] = \
                    (model['obj_detection_dist_edges'][1:] + \
                     model['obj_detection_dist_edges'][:-1])/2.

                for key,val in model.iteritems():
                    setattr(self,key,val)
                (self.nr,self.nc) = self.bg_mu.shape

            finally:
                if params.interactive:
                    wx.EndBusyCursor()
                    
        else:

            # parameters
            self.LoG_hsize = LoG_hsize
            self.LoG_sigma = LoG_sigma
            self.LoG_nonmaxsup_sz = LoG_nonmaxsup_sz
            self.LoG_thresh = LoG_thresh
            self.difftype = difftype
            self.obj_detection_dist_nbins = obj_detection_dist_nbins
            self.isback_dilation_hsize = isback_dilation_hsize
            self.fg_min_sigma = fg_min_sigma
            self.bg_min_sigma = bg_min_sigma
            self.min_always_bg_mask_frac = min_always_bg_mask_frac
            self.min_frac_cc_covered = min_frac_cc_covered
            self.min_area_cc_covered = min_area_cc_covered
            self.backsub_n_bg_std_thresh = backsub_n_bg_std_thresh
            self.backsub_n_bg_std_thresh_low = backsub_n_bg_std_thresh_low
            self.backsub_do_use_morphology = backsub_do_use_morphology
            self.backsub_opening_radius = backsub_opening_radius
            self.backsub_closing_radius = backsub_closing_radius

            # labeled movies
            # for now, we will assume that every frame of each movie
            # is labeled
            self.movienames = movienames
            self.annnames = annnames
    
            self.nmovies = len(movienames)
            if len(self.annnames) != self.nmovies:
                print "Number of annotation and movie files must match"
                raise Exception, "Number of annotation and movie files must match"
    
            # get movie size
            self.movie = movies.Movie( self.movienames[0], interactive=False )
            self.nr = self.movie.get_height()
            self.nc = self.movie.get_width()
            params.GRID.setsize((self.nr,self.nc))
    
            # initialize models
            # initialize each model
            self.init_fg_model()
            self.init_bg_model()
            self.init_obj_detection()
            self.init_always_bg_mask()
        
    def est_marginals(self):
        """
        est_marginals()

        This function calls functions to estimate the following 
        marginals from the training data:
        p(pixel appearance | foreground),
        p(pixel appearance | background), 
        p(object detection features | foreground), and
        p(object detection features | background)
        """

        for i in range(self.nmovies):

            if DEBUG: print "Computing per-movie model for movie %d"%i
            self.compute_models_permovie(i)

        # pool data for foreground model
        if DEBUG: print "Computing foreground appearance marginal"
        self.est_fg_appearance_marginal()

        # pool data for foreground model
        if DEBUG: print "Computing background appearance marginal"
        self.est_bg_appearance_marginal()

        # normalize histograms of distance to object detections
        if DEBUG: print "Computing object detection distance marginal"
        self.est_obj_detection_marginal()
        
        if DEBUG: print "Estimating always_bg_mask"
        self.est_always_bg_mask()
        
    def est_fg_appearance_marginal(self):
        """
        est_fg_appearance_marginal()

        This function estimates
        p(pixel appearance | foreground)
        from the training data. It uses pool_data to find an estimate
        of the pixel appearance mean and standard deviation for 
        foreground pixels observed near each pixel location. 
        """

        # radius we end up pooling to
        self.fg_poolradius = num.zeros((self.nr,self.nc))

        pool_data(self.fg_nsamples_px,self.fg_mu_px,
                  self.fg_sigma_px,
                  params.prior_fg_nsamples_pool,
                  params.prior_fg_pool_radius_factor,
                  self.fg_nsamples,self.fg_mu,
                  self.fg_sigma,self.fg_poolradius,
                  self.fg_min_sigma)

        self.fg_log_Z = .5*num.log(2*num.pi) + num.log(self.fg_sigma)

    def est_bg_appearance_marginal(self):
        """
        est_bg_appearance_marginal()

        This function estimates
        p(pixel appearance | background)
        from the training data. It uses pool_data to find an estimate
        of the pixel appearance mean and standard deviation for 
        background pixels observed near each pixel location. 
        """

        # radius we end up pooling to
        self.bg_poolradius = num.zeros((self.nr,self.nc))

        pool_data(self.bg_nsamples_px,self.bg_mu_px,
                  self.bg_sigma_px,
                  params.prior_bg_nsamples_pool,
                  params.prior_bg_pool_radius_factor,
                  self.bg_nsamples,self.bg_mu,
                  self.bg_sigma,self.bg_poolradius,
                  self.bg_min_sigma)

        # log( sqrt(2*pi) * sigma ) = .5*log(2*pi) + log(sigma)
        self.bg_log_Z = .5*num.log(2*num.pi) + num.log(self.bg_sigma)    

    def est_obj_detection_marginal(self):
        """
        est_obj_detection_marginal()

        This function estimates
        p(dist to nearest object detection|foreground) and
        p(dist to nearest object detection|background).

        It uses the histogrammed counts as an estimate. 
        """

        # normalize histograms
        Z = num.sum(self.obj_detection_dist_counts_fg)
        self.obj_detection_dist_frac_fg = self.obj_detection_dist_counts_fg / Z
        Z = num.sum(self.obj_detection_dist_counts_bg)
        self.obj_detection_dist_frac_bg = self.obj_detection_dist_counts_bg / Z

    def compute_models_permovie( self, i ):
        """
        compute_models_permovie( i )
        
        For movienames[i], this function samples frames from throughout
        the movie and computes the data structures necessary for 
        estimating the marginal distributions. 
        """

        # open movie
        self.movie = movies.Movie( self.movienames[i], params.interactive )

        # background model
        self.bg_imgs = bg.BackgroundCalculator(self.movie)

        # open annotation
        self.trx = annot.AnnotationFile( self.annnames[i], bg_imgs=self.bg_imgs, 
                                        doreadtrx=True, readonly=True )


        self.setup_backsub()

        # choose frames to learn from: for now, assume all frames are tracked
        self.firstframe = self.trx.firstframetracked
        self.lastframe = self.trx.lastframetracked

        self.framessample = num.unique(num.round(num.linspace(self.firstframe+1,
                                                              self.lastframe,
                                                              params.prior_nframessample)).astype(int))
        # update the always_bg mask data structures for this movie
        self.update_always_bg_mask()

        if DEBUG: print "Collecting foreground and background data..."

        for j in range(len(self.framessample)):

            # read in the sampled frame
            self.frame = self.framessample[j]
            self.im,self.timestamp = self.movie.get_frame(self.frame)
            self.trxcurr = self.trx.get_frame(self.frame)

            if DEBUG: print "Collecting data for frame %d"%self.frame

            # create an image which is True inside the ellipses, False outside
            self.isfore_mask()

            # update all the observation models
            self.update_fg_appearance_marginal()
            self.update_bg_appearance_marginal()
            self.update_obj_detection_marginal()

        self.movie.close()
        self.movie = None
        self.trx.close()
        self.trx = None
        self.bg_imgs = None
                                
    def compute_log_lik_appearance_given_fore(self,r0=0,r1=num.inf):
        """
        compute_log_lik_appearance_given_fore()
        
        computes the log likelihood of each pixel appearance in
        self.im given that the pixel is foreground. 
        """
        r1 = min(r1,self.nr)
        d2 = (self.im[r0:r1] - self.fg_mu[r0:r1])**2 / (2*self.fg_sigma[r0:r1]**2)
        self.log_lik_appearance_given_fore = -d2 - self.fg_log_Z[r0:r1]

    def compute_log_lik_appearance_given_back(self,r0=0,r1=num.inf):
        """
        compute_log_lik_appearance_given_back()
        
        computes the log likelihood of each pixel appearance in
        self.im given that the pixel is background. 
        """
        
        r1 = min(r1,self.nr)

        d2 = (self.im[r0:r1] - self.bg_mu[r0:r1])**2 / (2*self.bg_sigma[r0:r1]**2)
        self.log_lik_appearance_given_back = -d2 - self.bg_log_Z[r0:r1]

    def compute_log_lik_dist_obj(self,r0=0,r1=num.inf):
        """
        compute_log_lik_dist_obj()
        
        computes the log likelihood of the distance to the nearest object detection
        for each pixel in the image self.im given that the pixel is in foreground and
        given that the pixel is in background. 
        """
        
        r1 = min(r1,self.nr)
        
        # detect objects in the current image
        self.obj_detect(r0=r0,r1=r1)

        # compute distances to detections
        morph.distance_transform_edt(~self.isobj,
                                     distances=self.obj_detection_dist[r0:r1])
        nr = r1 - r0

        # find which bin each distance falls in
        idx = num.digitize(num.reshape(self.obj_detection_dist[r0:r1],(nr*self.nc,)),self.obj_detection_dist_edges)
        idx.shape = (nr,self.nc)

        # lookup probabilities for each bin idx
        self.log_lik_dist_obj_given_fore = num.log(self.obj_detection_dist_frac_fg[idx])
        self.log_lik_dist_obj_given_back = num.log(self.obj_detection_dist_frac_bg[idx])

    def compute_log_lik(self,r0=0,r1=num.inf):

        self.compute_log_lik_appearance_given_fore(r0=r0,r1=r1)
        self.compute_log_lik_appearance_given_back(r0=r0,r1=r1)
        self.compute_log_lik_dist_obj(r0=r0,r1=r1)
        self.log_lik_given_fore = self.log_lik_appearance_given_fore + \
            self.log_lik_dist_obj_given_fore
        self.log_lik_given_back = self.log_lik_appearance_given_back + \
            self.log_lik_dist_obj_given_back
            
    def compute_log_lik_ratio(self,r0=0,r1=num.inf):
        
        self.compute_log_lik(r0=r0,r1=r1)
        self.log_lik_ratio = self.log_lik_given_fore - self.log_lik_given_back
        return self.log_lik_ratio
            
    def setup_backsub(self):

        params.n_bg_std_thresh = self.backsub_n_bg_std_thresh
        params.n_bg_std_thresh_low = self.backsub_n_bg_std_thresh_low
        params.do_use_morphology = self.backsub_do_use_morphology
        params.opening_radius = self.backsub_opening_radius
        params.closing_radius = self.backsub_closing_radius
        if params.do_use_morphology:
            self.bg_imgs.opening_struct = self.bg_imgs.create_morph_struct(params.opening_radius)
            self.bg_imgs.closing_struct = self.bg_imgs.create_morph_struct(params.closing_radius)

    def backsub(self, im, stamp):

        return self.bg_imgs.sub_bg(im, stamp)

    def isfore_mask(self):

        self.isfore = num.zeros((self.nr,self.nc),dtype=bool)
        idx = num.zeros((self.nr,self.nc),dtype=bool)
        #print 'backsub_n_bg_std_thresh = ' + str(params.n_bg_std_thresh)
        #print 'backsub_n_bg_std_thresh should be ' + str(self.backsub_n_bg_std_thresh)
        #print 'backsub_n_bg_std_thresh_low = ' + str(params.n_bg_std_thresh_low)
        #print 'backsub_do_use_morphology = ' + str(params.do_use_morphology)
        #print 'backsub_opening_radius = ' + str(params.opening_radius)
        #print 'backsub_closing_radius = ' + str(params.closing_radius)
        #print 'min_frac_cc_covered = ' + str(self.min_frac_cc_covered)
        #print 'min_area_cc_covered = ' + str(self.min_area_cc_covered)
        (self.dfore,self.bw,self.cc,self.ncc) = self.backsub()

        # area of each connected component
        areas = num.zeros(self.ncc)
        for i in range(self.ncc):
            areas[i] = num.alen(num.flatnonzero(self.cc==i+1))

        for ell in self.trxcurr.itervalues():

            # find pixels inside this ellipse
            (r1,r2,c1,c2) = est.getboundingboxtight(ell,(self.nr,self.nc))
            isfore1 = est.ellipsepixels(ell,num.array([r1,r2,c1,c2]))

            # which cc(s) does this correspond to?
            r,c = isfore1.nonzero()
            counts,edges = num.histogram(self.cc[int(r1)+r,int(c1)+c],num.arange(1,self.ncc+2))
            fraccovered = counts / areas
            iscovered = fraccovered >= self.min_frac_cc_covered
            mode = num.argmax(counts)
            if counts[mode] > self.min_area_cc_covered:
                iscovered[mode] = True
            #print "counts = " + str(counts)
            #print "areas = " + str(areas)
            #print "fraccovered = " + str(fraccovered)
            #print "mode = " + str(mode)

            # set isfore to True for these ccs
            for i in num.flatnonzero(iscovered):
                self.isfore[self.cc==i+1] = True

            #h0 = plt.subplot(131)
            #plt.imshow(self.im)
            #plt.hold(True)
            #plt.plot(ell.x,ell.y,'ro')
            #plt.axis('image')
            #plt.title('image')
            #plt.subplot(132,sharex=h0,sharey=h0)
            #plt.title('cc')
            #plt.imshow(self.cc)
            #plt.subplot(133,sharex=h0,sharey=h0)
            #isforecurr = num.zeros((self.nr,self.nc),dtype=bool)
            #for i in num.flatnonzero(iscovered):
            #    isforecurr[self.cc==i+1] = True
            #plt.imshow(isforecurr)
            #if num.any(isforecurr):
            #    rcurr,ccurr = num.where(isforecurr)
            #    r0curr = num.min(rcurr)-1
            #    r1curr = num.max(rcurr)+1
            #    c0curr = num.min(ccurr)-1
            #    c1curr = num.max(ccurr)+1
            #    plt.hold(True)
            #    plt.plot(num.array([c0curr,c0curr,c1curr,c1curr,c0curr]),
            #             num.array([r0curr,r1curr,r1curr,r0curr,r0curr]),'w-')
            #plt.title('isfore curr')
            #plt.show()


        # compute isback by dilating and flipping
        self.isback = morph.binary_dilation(self.isfore,self.dilation_struct) == False

        h0 = plt.subplot(151)
        plt.imshow(self.im)
        plt.axis('tight')
        #plt.axis('image')
        plt.title('image')
        plt.subplot(152,sharex=h0,sharey=h0)
        plt.title('cc')
        plt.imshow(self.cc)
        plt.axis('tight')
        plt.subplot(153,sharex=h0,sharey=h0)
        plt.imshow(self.isfore)
        plt.title('isfore')
        plt.axis('tight')
        plt.subplot(154,sharex=h0,sharey=h0)
        plt.imshow(self.isback)
        plt.title('isback')
        plt.axis('tight')
        plt.subplot(155,sharex=h0,sharey=h0)
        plt.imshow(num.logical_and(self.bw,self.isfore==False))
        plt.title('ignored fg pixels')
        plt.axis('tight')
        plt.show()
            
    def isfore_mask_old(self):
        """
        isfore_mask()

        Computes a mask self.isfore which is 1 inside of ellipses
        in the current frame and 0 outside. Also computes a mask
        self.isback which is an eroded version of the complement. 
        """

        self.isfore = num.zeros((self.nr,self.nc),dtype=bool)

        # loop through each ellipse
        for ell in self.trxcurr.itervalues():

            # find pixels inside this ellipse
            (r1,r2,c1,c2) = est.getboundingboxtight(ell,(self.nr,self.nc))
            isfore1 = est.ellipsepixels(ell,num.array([r1,r2,c1,c2]))
            self.isfore[r1:r2,c1:c2] = isfore1

        # compute isback by dilating and flipping
        self.isback = morph.binary_dilation(self.isfore,self.dilation_struct) == False
        
    def init_fg_model(self):
        """
        init_fg_model()

        Initializes data structures for the foreground model.
        """
        
        self.fg_nsamples_px = num.zeros((self.nr,self.nc))
        self.fg_mu_px = num.zeros((self.nr,self.nc))
        self.fg_sigma_px = num.zeros((self.nr,self.nc))

        self.fg_nsamples = num.zeros((self.nr,self.nc))
        self.fg_mu = num.zeros((self.nr,self.nc))
        self.fg_sigma = num.zeros((self.nr,self.nc))

        # for morphology
        self.dilation_struct = num.ones((self.isback_dilation_hsize,self.isback_dilation_hsize),bool)


    def init_bg_model(self):
        """
        init_bg_model()

        Initializes data structures for the background model.
        """
        
        self.bg_nsamples_px = num.zeros((self.nr,self.nc))
        self.bg_mu_px = num.zeros((self.nr,self.nc))
        self.bg_sigma_px = num.zeros((self.nr,self.nc))

        self.bg_nsamples = num.zeros((self.nr,self.nc))
        self.bg_mu = num.zeros((self.nr,self.nc))
        self.bg_sigma = num.zeros((self.nr,self.nc))

    def update_fg_appearance_marginal(self):
        """
        update_fg_appearance_marginal()

        Update the foreground pixel appearance data structures 
        based on the current frame. 
        """
        
        # increment number of samples in foreground pixels
        self.fg_nsamples_px[self.isfore] += 1
        # add to mean
        self.fg_mu_px[self.isfore] += self.im[self.isfore]
        # add to sigma
        self.fg_sigma_px[self.isfore] += self.im[self.isfore]**2

    def update_bg_appearance_marginal(self):
        """
        update_bg_appearance_marginal()

        Update the background pixel appearance data structures 
        based on the current frame.
        """

        # increment number of samples in background pixels
        self.bg_nsamples_px[self.isback] += 1
        # add to mean
        self.bg_mu_px[self.isback] += self.im[self.isback]
        # add to sigma
        self.bg_sigma_px[self.isback] += self.im[self.isback]**2

    def update_obj_detection_marginal(self):
        """
        update_obj_detection_marginal()

        Update the object detection data structures based on the
        current frame. 
        """        

        # detect objects in the current image
        self.obj_detect()

        # compute distances to detections
        morph.distance_transform_edt(~self.isobj,distances=self.obj_detection_dist)

        # histogram distances for foreground pixels
        counts,edges = num.histogram(self.obj_detection_dist[self.isfore],
                               bins=self.obj_detection_dist_edges)
        self.obj_detection_dist_counts_fg += counts

        # histogram distances for background pixels
        counts,edges = num.histogram(self.obj_detection_dist[self.isback],
                               bins=self.obj_detection_dist_edges)
        self.obj_detection_dist_counts_bg += counts
                      
                      
    def obj_detect(self,r0=0,r1=num.inf):
        """
        obj_detect()

        Detect objects in the current frame. For now, this is just
        LoG filtering. 
        """

        r1 = min(r1,self.nr)

        self.LoGfilter(r0=r0,r1=r1)
        
        if hasattr(self,'always_bg_mask') and self.always_bg_mask is not None:
            self.isobj[self.always_bg_mask[r0:r1]] = False

    def init_obj_detection(self):
        """
        init_obj_detection()

        Initialize object detection data structures. 
        """

        # create LoG filter
        self.make_LoG_filter()

        # output of LoG filtering
        self.LoG_fil_out = num.zeros((self.nr,self.nc))
        # output of nonmaximal suppression
        self.LoG_fil_max = num.zeros((self.nr,self.nc))

        # output of dist transform
        self.obj_detection_dist = num.zeros((self.nr,self.nc))

        # edges used for histogramming distance to object detections. use bins 
        # centered at 0:1:nbins/2, log spacing between nbins/2 and maximum distance 
        # in the image
        n1 = int(num.floor(self.obj_detection_dist_nbins/2))
        n2 = self.obj_detection_dist_nbins - n1
        edges1 = num.arange(-.5,n1-.5)
        edges2 = num.unique(num.round(num.exp(num.linspace(num.log(n1+.5),num.log(num.sqrt(self.nr**2+self.nc**2)+.5),n2+1))-.5)+.5)
        self.obj_detection_dist_edges = num.concatenate((edges1,edges2))
        self.obj_detection_dist_nbins = len(self.obj_detection_dist_edges) - 1
        self.obj_detection_dist_centers = \
            (self.obj_detection_dist_edges[1:] + \
                 self.obj_detection_dist_edges[:-1])/2.

        # histograms of distances to object detections for foreground and background:
        # initialize to 1 so that there is never a 0 probability
        self.obj_detection_dist_counts_fg = num.ones(self.obj_detection_dist_nbins)
        self.obj_detection_dist_counts_bg = num.ones(self.obj_detection_dist_nbins)

    def init_always_bg_mask(self):
        self.always_bg_mask_count = num.zeros((self.nr,self.nc))
        self.n_always_bg_mask_samples = 0

    def make_LoG_filter(self):
        """
        make_LoG_filter()

        Initialize the LoG filter
        """
        
        # Gaussian
        radius = (self.LoG_hsize-1)/2
        [x,y] = num.meshgrid(num.arange(-radius,radius),
                             num.arange(-radius,radius))
        z = -(x**2+y**2) / (2*self.LoG_sigma**2)
        gau = num.exp(z)

        # sum to 1
        Z = num.sum(gau)
        if Z > 0:
            gau = gau / Z

        # Laplacian
        lap = (x**2 + y**2 - 2*self.LoG_sigma**2) / self.LoG_sigma**4

        self.LoG_fil = lap*gau

        # make sum to zero
        self.LoG_fil = self.LoG_fil - num.mean(self.LoG_fil)

        # zero out small values
        absfil = num.abs(self.LoG_fil)
        self.LoG_fil[absfil<.00001*num.max(absfil)] = 0

        # for light flies on a dark background, flip
        if self.difftype == 'lightfgdarkbg':
            self.LoG_fil = -self.LoG_fil
        elif self.difftype == 'darkfglightbg':
            pass
        else:
            self.LoG_fil = num.abs(self.LoG_fil)
        

    def LoGfilter(self,r0=0,r1=num.inf):
        """
        LoGfilter()

        Apply LoG filtering to the current frame to detect blobs. 
        """
        
        r1 = min(r1,self.nr)
        
        # LoG filter
        if not hasattr(self,'LoG_fil_out'):
            # output of LoG filtering
            self.LoG_fil_out = num.zeros((self.nr,self.nc))
            
        if not hasattr(self,'LoG_fil_max'):
            # output of nonmaximal suppression
            self.LoG_fil_max = num.zeros((self.nr,self.nc))

        if not hasattr(self,'obj_detection_dist'):
            # output of dist transform
            self.obj_detection_dist = num.zeros((self.nr,self.nc))
            
        # pad borders
        off = max(self.LoG_hsize,self.LoG_nonmaxsup_sz) 
        r00 = max(0,r0-off)
        r11 = min(self.nr,r1+off)

        filters.correlate(self.im[r00:r11].astype(float),
                          self.LoG_fil,
                          output=self.LoG_fil_out[r00:r11],
                          mode='nearest')

        # depending on difftype, we will only look for positive, negative or both values
        if self.difftype == 'other':
            self.LoG_fil_out[r00:r11] = num.abs(self.LoG_fil_out[r00:r11])

        # non-maximal suppression + threshold
        filters.maximum_filter(self.LoG_fil_out[r00:r11],size=self.LoG_nonmaxsup_sz,
                               output=self.LoG_fil_max[r00:r11],mode='constant',cval=0)
        self.isobj = num.logical_and(self.LoG_fil_out[r0:r1] == self.LoG_fil_max[r0:r1],
                                     self.LoG_fil_out[r0:r1] >= self.LoG_thresh)
                                     
    def update_always_bg_mask(self):

        # TODO: use calibration
        
        # current always bg mask
        always_bg_mask = self.bg_imgs.isarena == False
        
        # count number of times each pixel is always background
        self.always_bg_mask_count += always_bg_mask.astype(float)
        self.n_always_bg_mask_samples += 1

    def est_always_bg_mask(self):
        
        self.always_bg_mask_frac = self.always_bg_mask_count / self.n_always_bg_mask_samples
        self.always_bg_mask = self.always_bg_mask_frac >= self.min_always_bg_mask_frac        
    
    def save_mat(self,matFileName):
        
        out = dict()
        if hasattr(self,'fg_mu'):
            out['fg_mu'] = self.fg_mu
        if hasattr(self,'fg_sigma'):
            out['fg_sigma'] = self.fg_sigma
        if hasattr(self,'fg_log_Z'):
            out['fg_log_Z'] = self.fg_log_Z
        if hasattr(self,'bg_mu'):
            out['bg_mu'] = self.bg_mu
        if hasattr(self,'bg_sigma'):
            out['bg_sigma'] = self.bg_sigma
        if hasattr(self,'bg_log_Z'):
            out['bg_log_Z'] = self.bg_log_Z
        if hasattr(self,'obj_detection_dist_frac_fg'):
            out['obj_detection_dist_frac_fg'] = self.obj_detection_dist_frac_fg
        if hasattr(self,'obj_detection_dist_frac_bg'):
            out['obj_detection_dist_frac_bg'] = self.obj_detection_dist_frac_bg
        if hasattr(self,'obj_detection_dist_edges'):
            out['obj_detection_dist_edges'] = self.obj_detection_dist_edges
        if hasattr(self,'obj_detection_dist_centers'):
            out['obj_detection_dist_centers'] = self.obj_detection_dist_centers
        if hasattr(self,'always_bg_mask_frac'):
            out['always_bg_mask_frac'] = self.always_bg_mask_frac
        if hasattr(self,'always_bg_mask'):
            out['always_bg_mask'] = self.always_bg_mask

        scipy.io.savemat( matFileName, out, oned_as='column' )

    def save(self,outputFileName):
        
        fid = open(outputFileName,"w")
        
        out = dict()
        if hasattr(self,'LoG_hsize'):
            out['LoG_hsize'] = self.LoG_hsize
        if hasattr(self,'LoG_sigma'):
            out['LoG_sigma'] = self.LoG_sigma
        if hasattr(self,'LoG_nonmaxsup_sz'):
            out['LoG_nonmaxsup_sz'] = self.LoG_nonmaxsup_sz
        if hasattr(self,'LoG_thresh'):
            out['LoG_thresh'] = self.LoG_thresh
        if hasattr(self,'difftype'):
            out['difftype'] = self.difftype
        if hasattr(self,'obj_detection_dist_nbins'):
            out['obj_detection_dist_nbins'] = self.obj_detection_dist_nbins
        if hasattr(self,'isback_dilation_hsize'):
            out['isback_dilation_hsize'] = self.isback_dilation_hsize
        if hasattr(self,'fg_min_sigma'):
            out['fg_min_sigma'] = self.fg_min_sigma
        if hasattr(self,'bg_min_sigma'):
            out['bg_min_sigma'] = self.bg_min_sigma
        if hasattr(self,'min_always_bg_mask_frac'):
            out['min_always_bg_mask_frac'] = self.min_always_bg_mask_frac
        if hasattr(self,'min_frac_cc_covered'):
            out['min_frac_cc_covered'] = self.min_frac_cc_covered
        if hasattr(self,'min_area_cc_covered'):
            out['min_area_cc_covered'] = self.min_area_cc_covered
        if hasattr(self,'backsub_n_bg_std_thresh'):
            out['backsub_n_bg_std_thresh'] = self.backsub_n_bg_std_thresh
        if hasattr(self,'backsub_n_bg_std_thresh_low'):
            out['backsub_n_bg_std_thresh_low'] = self.backsub_n_bg_std_thresh_low
        if hasattr(self,'backsub_do_use_morphology'):
            out['backsub_do_use_morphology'] = self.backsub_do_use_morphology
        if hasattr(self,'backsub_opening_radius'):
            out['backsub_opening_radius'] = self.backsub_opening_radius
        if hasattr(self,'backsub_closing_radius'):
            out['backsub_closing_radius'] = self.backsub_closing_radius
        if hasattr(params,'prior_nframessample'):
            out['prior_nframessample'] = params.prior_nframessample
        if hasattr(params,'prior_fg_nsamples_pool'):
            out['prior_fg_nsamples_pool'] = params.prior_fg_nsamples_pool
        if hasattr(params,'prior_bg_nsamples_pool'):
            out['prior_bg_nsamples_pool'] = params.prior_bg_nsamples_pool
        if hasattr(params,'prior_fg_pool_radius_factor'):
            out['prior_fg_pool_radius_factor'] = params.prior_fg_pool_radius_factor
        if hasattr(params,'prior_bg_pool_radius_factor'):
            out['prior_bg_pool_radius_factor'] = params.prior_bg_pool_radius_factor
        if hasattr(self,'movienames'):
            out['movienames'] = self.movienames
        if hasattr(self,'annnames'):
            out['annnames'] = self.annnames
        if hasattr(self,'fg_mu'):
            out['fg_mu'] = self.fg_mu
        if hasattr(self,'fg_sigma'):
            out['fg_sigma'] = self.fg_sigma
        if hasattr(self,'fg_log_Z'):
            out['fg_log_Z'] = self.fg_log_Z
        if hasattr(self,'bg_mu'):
            out['bg_mu'] = self.bg_mu
        if hasattr(self,'bg_sigma'):
            out['bg_sigma'] = self.bg_sigma
        if hasattr(self,'bg_log_Z'):
            out['bg_log_Z'] = self.bg_log_Z
        if hasattr(self,'obj_detection_dist_frac_fg'):
            out['obj_detection_dist_frac_fg'] = self.obj_detection_dist_frac_fg
        if hasattr(self,'obj_detection_dist_frac_bg'):
            out['obj_detection_dist_frac_bg'] = self.obj_detection_dist_frac_bg
        if hasattr(self,'obj_detection_dist_edges'):
            out['obj_detection_dist_edges'] = self.obj_detection_dist_edges
        if hasattr(self,'LoG_fil'):
            out['LoG_fil'] = self.LoG_fil
        if hasattr(self,'always_bg_mask_frac'):
            out['always_bg_mask_frac'] = self.always_bg_mask_frac
        if hasattr(self,'always_bg_mask'):
            out['always_bg_mask'] = self.always_bg_mask

        pickle.dump(out,fid)
        fid.close()


    def set_movie(self,moviename):

        self.movie = movies.Movie( moviename, params.interactive )
        self.nr = self.movie.get_height()
        self.nc = self.movie.get_width()


    def show(self):

        plt.subplot(231)
        plt.imshow(self.fg_mu,cmap=cm.gray,vmin=0,vmax=255)
        plt.colorbar()
        plt.title("foreground mean")
    
        plt.subplot(234)
        plt.imshow(self.fg_sigma)
        plt.colorbar()
        plt.title("foreground standard deviation")
    
        plt.subplot(232)
        plt.imshow(self.bg_mu,cmap=cm.gray,vmin=0,vmax=255)
        plt.colorbar()
        plt.title("background mean")
    
        plt.subplot(235)
        plt.imshow(self.bg_sigma)
        plt.colorbar()
        plt.title("background standard deviation")
        
        plt.subplot(233)
        plt.imshow(self.always_bg_mask_frac)
        plt.colorbar()
        plt.title("always bg mask frac")
    
        plt.subplot(236)
        plt.imshow(self.always_bg_mask,cmap=cm.gray)
        plt.colorbar()
        plt.title("always bg mask")
    
        plt.figure()
        plt.plot(self.obj_detection_dist_centers,self.obj_detection_dist_frac_bg,'k.-',
                 self.obj_detection_dist_centers,self.obj_detection_dist_frac_fg,'r.-')
        plt.legend(('bg','fg'))
        plt.xlabel('Dist to obj detection')
        plt.ylabel('P(dist | label)')
    
        plt.show()
        
    def showtest(self,moviename=None,moviei=0,nframessample=4,cmin=-num.infty,cmax=num.infty):
        
        if moviename is None:
            moviename = self.movienames[moviei]
            
        self.movie = movies.Movie( moviename, params.interactive )
        self.nr = self.movie.get_height()
        self.nc = self.movie.get_width()

        framessample = num.unique(num.round(num.linspace(0,self.movie.get_n_frames()-1,nframessample)).astype(int))
        
        nr = nframessample
        nc = 5
        
        plt.figure(figsize=(15,15))
        vmin_noninf = num.zeros(nc)
        vmin_noninf[:] = num.infty
        vmax_noninf = num.zeros(nc)
        vmax_noninf[:] = -num.infty
        vmin = num.zeros(nc)
        vmin[:] = num.infty
        vmax = num.zeros(nc)
        vmax[:] = -num.infty
        
        hims = []
        for i in range(nframessample):
            
            self.frame = framessample[i]
            print 'Frame %d (%d/%d)'%(self.frame,i+1,nframessample)
            self.im,self.timestamp = self.movie.get_frame(self.frame)
            log_lik_ratio = self.compute_log_lik_ratio()
            
            j = 1
            h0 = plt.subplot(nr,nc,nc*i+j)
            tmp = self.log_lik_appearance_given_fore.copy()
            tmp[self.always_bg_mask] = num.nan
            him = []
            him.append(plt.imshow(tmp))
            plt.axis('off')
            plt.axis('image')
            plt.title('ll(intensity|fg), %d'%self.frame)
            (vmin,vmax,vmin_noninf,vmax_noninf) = update_clim(self.log_lik_appearance_given_fore,vmin,vmax,vmin_noninf,vmax_noninf,j)
            plt.colorbar()
            
            j += 1
            plt.subplot(nr,nc,nc*i+j,sharex=h0,sharey=h0)
            tmp = self.log_lik_appearance_given_back.copy()
            tmp[self.always_bg_mask] = num.nan
            him.append(plt.imshow(tmp))
            plt.axis('off')
            plt.title('ll(intensity|bg), %d'%self.frame)
            (vmin,vmax,vmin_noninf,vmax_noninf) = update_clim(self.log_lik_appearance_given_back,vmin,vmax,vmin_noninf,vmax_noninf,j)
            plt.colorbar()

            j += 1
            plt.subplot(nr,nc,nc*i+j,sharex=h0,sharey=h0)
            tmp = self.log_lik_dist_obj_given_fore.copy()
            tmp[self.always_bg_mask] = num.nan
            him.append(plt.imshow(tmp))
            plt.axis('off')
            plt.title('ll(distobj|fg), %d'%self.frame)
            (vmin,vmax,vmin_noninf,vmax_noninf) = update_clim(self.log_lik_dist_obj_given_fore,vmin,vmax,vmin_noninf,vmax_noninf,j)
            plt.colorbar()

            j += 1
            plt.subplot(nr,nc,nc*i+4,sharex=h0,sharey=h0)
            tmp = self.log_lik_dist_obj_given_back.copy()
            tmp[self.always_bg_mask] = num.nan
            him.append(plt.imshow(tmp))
            plt.axis('off')
            plt.title('ll(distobj|bg), %d'%self.frame)
            (vmin,vmax,vmin_noninf,vmax_noninf) = update_clim(self.log_lik_dist_obj_given_back,vmin,vmax,vmin_noninf,vmax_noninf,j)
            plt.colorbar()
            
            j += 1
            plt.subplot(nr,nc,nc*i+j,sharex=h0,sharey=h0)
            tmp = log_lik_ratio.copy()
            tmp[self.always_bg_mask] = num.nan
            him.append(plt.imshow(tmp))
            plt.axis('off')
            plt.title('ll(im|fg) - ll(im|bg), %d'%self.frame)
            (vmin,vmax,vmin_noninf,vmax_noninf) = update_clim(log_lik_ratio,vmin,vmax,vmin_noninf,vmax_noninf,j)
            plt.colorbar()
            
            hims.append(him)
            plt.draw()

        vmin = num.maximum(vmin,cmin)
        vmin_noninf = num.maximum(vmin_noninf,cmin)
        vmax = num.minimum(vmax,cmax)
        vmax_noninf = num.minimum(vmax_noninf,cmax)
        dv = vmax_noninf - vmin_noninf
        idx = vmin < vmin_noninf
        vmin_noninf[idx] = vmin_noninf[idx] - dv[idx]*.025
        idx = vmax > vmax_noninf
        vmax_noninf[idx] = vmax_noninf[idx] + dv[idx]*.025

        for j in range(nc):
            for i in range(nr):
                hims[i][j].set_clim(vmin_noninf[j],vmax_noninf[j])
                
        print "Done."
        
        plt.show()
            
def update_clim(x,vmin,vmax,vmin_noninf,vmax_noninf,j):
    
    vmin[j-1] = min(num.min(x),vmin[j-1])
    vmax[j-1] = max(num.max(x),vmax[j-1])
    vmin_noninf[j-1] = min(num.min(x[False == num.isinf(x)]),vmin_noninf[j-1])
    vmax_noninf[j-1] = max(num.max(x[False == num.isinf(x)]),vmax_noninf[j-1])

    return (vmin,vmax,vmin_noninf,vmax_noninf)
            
    
def create_morph_struct(radius):
    """
    create_morph_struct(radius)

    Create disk structure for morphological filtering. 
    
    """
    if radius <= 0:
        s = False
    elif radius == 1:
        s = num.ones((2,2),bool)
    else:
        s = morph.generate_binary_structure(2,1)
        s = morph.iterate_structure(s,radius)
    return s
    
    
def integral_image_sum(int_a,r0,r1,c0,c1):
    """
    integral_image_sum(int_a,r0,r1,c0,c1)

    Compute the sum of all values in the box [r0,r1],[c0,c1]
    using the pref-computed integral image int_a. 
    """
    
    s = int_a[r1,c1]
    idx = c0 > 0
    s[idx] -= int_a[r1[idx],c0[idx]-1]
    idx = r0 > 0
    s[idx] -= int_a[r0[idx]-1,c1[idx]]
    idx = num.logical_and(r0 > 0,c0 > 0)
    s[idx] += int_a[r0[idx]-1,c0[idx]-1]

    return s

def pool_data(nsamples_px,mu_px,sigma_px,nsamples_pool,pool_radius_factor,
              nsamples,mu,sigma,poolradius,min_sigma):

    """
    pool_data(nsamples_px,mu_px,sigma_px,nsamples_pool,pool_radius_factor,
    nsamples,mu,sigma,poolradius,min_sigma)

    At each pixel location, we want to sample nsamples_pool pixels 
    near the current location with the given label. We start by looking
    in a box of radius 1 (so the width and height of the box are 2*1 + 1)
    and count the number of training samples. If this is at least 
    nsamples_pool, then we compute the mean and standard deviation of
    the training samples, and this is the mean and standard deviation
    for the current pixel location. Otherwise, we increase the sample
    radius by the factor pool_radius_factor, and repeat. We continue
    in this way until enough samples are found or until the pooling
    radius exceeds maxradius. 

    Inputs:
    
    nsamples_px is an array of size [nr,nc], and stores the number 
    of training samples of the given label observed at each pixel 
    location. 
    mu_px is an array of size [nr,nc], and stores the mean of all
    the training samples of the given label observed at each pixel 
    location
    sigma_px is an array of size [nr,nc], and stores the std of all
    the training samples of the given label observed at each pixel 
    location
    nsamples_pool is the ideal number of samples we will take around
    each pixel location. 
    pool_radius_factor is the factor to increase the pooling radius
    by at each iteration. 
    min_sigma is the minimum standard deviation to be returned. 

    Modified inputs:

    nsamples is an array of size [nr,nc], and nsamples[i,j] is the
    number of samples collected for pixel location [i,j]. 
    mu is an array of size [nr,nc], and mu[i,j] is the estimated mean
    for pixel location [i,j]
    sigma is an array of size [nr,nc], and sigma[i,j] is the 
    estimated standard deviation for pixel location [i,j]
    poolradius is an array of size [nr,nc], and poolradius[i,j] is
    the radius that samples were taken over for pixel location [i,j]
    """

    nr = nsamples_px.shape[0]
    nc = nsamples_px.shape[1]

    # precompute integral image
    int_nsamples_px = num.cumsum(num.cumsum(nsamples_px,axis=0),axis=1)
    int_mu_px = num.cumsum(num.cumsum(mu_px,axis=0),axis=1)
    int_sigma_px = num.cumsum(num.cumsum(sigma_px,axis=0),axis=1)

    # whether we are done pooling
    notdone = num.ones((nr,nc),dtype=bool)

    radius = 1
    maxradius = max(nr,nc)
    while (radius <= maxradius) and notdone.any():

        # where do we need to continue incrementing radius?
        r,c = num.where(notdone)

        # current box
        r0 = num.maximum(0,r-radius)
        r1 = num.minimum(nr-1,r+radius)
        c0 = num.maximum(0,c-radius)
        c1 = num.minimum(nc-1,c+radius)

        # compute number of samples in each box
        nsamplescurr = integral_image_sum(int_nsamples_px,r0,r1,c0,c1)

        # which pixels are we now done for?
        newdone = nsamplescurr >= nsamples_pool

        if not newdone.any():
            # increase the radius
            radius = max(radius+1,int(round(radius*pool_radius_factor)))

            if DEBUG: print "no pixels newly with enough samples, incrementing radius to %d"%radius

            continue

        notdone[notdone] = ~newdone

        # for pixels that just finished
        r = r[newdone]
        c = c[newdone]
        r0 = r0[newdone]
        r1 = r1[newdone]
        c0 = c0[newdone]
        c1 = c1[newdone]

        # store the radius and number of samples
        poolradius[r,c] = radius
        nsamples[r,c] = nsamplescurr[newdone]

        # compute the average color
        mu[r,c] = integral_image_sum(int_mu_px,r0,r1,c0,c1) / \
            nsamples[r,c]

        # compute the color standard deviation
        sigma[r,c] = num.sqrt(integral_image_sum(int_sigma_px,r0,r1,c0,c1) / \
            nsamples[r,c])

        if DEBUG: print "n pixels added at radius %d: %d"%(radius,len(r))

        #if DEBUG:
        #    print "\nNew pixels at radius = %d:"%radius
        #    for i in range(len(r)):
        #        print "For pixel (%d,%d), radius = %d, mu = %f, sigma = %f, nsamples = %d"%\
        #          (r[i],c[i],radius,mu[r[i],c[i]],sigma[r[i],c[i]],
        #           nsamples[r[i],c[i]])

        # increase the radius
        radius = max(radius+1,int(round(radius*pool_radius_factor)))

    # make sure standard deviations are large enough
    sigma = num.maximum(sigma,min_sigma)

def test1():
    """
    test1()

    Compute experiment bg and fg models for a set of movies set
    at the start of the function, and plot the results. 
    """

    movienames = ['E:\Data\FlyBowl\Test1\GMR_13B06_AE_01_TrpA_Rig1Plate01BowlA_20101007T161822/movie.ufmf']
    annnames = ['E:\Data\FlyBowl\Test1\GMR_13B06_AE_01_TrpA_Rig1Plate01BowlA_20101007T161822/movie.ufmf.ann',]

    self = ExpBGFGModel(movienames,annnames)
    
    self.est_marginals()

    plt.subplot(231)
    plt.imshow(self.fg_mu,cmap=cm.gray,vmin=0,vmax=255)
    plt.colorbar()
    plt.title("foreground mean")

    plt.subplot(234)
    plt.imshow(self.fg_sigma)
    plt.colorbar()
    plt.title("foreground standard deviation")

    plt.subplot(232)
    plt.imshow(self.bg_mu,cmap=cm.gray,vmin=0,vmax=255)
    plt.colorbar()
    plt.title("background mean")

    plt.subplot(235)
    plt.imshow(self.bg_sigma)
    plt.colorbar()
    plt.title("background standard deviation")
    
    plt.subplot(233)
    plt.imshow(self.always_bg_mask_frac)
    plt.colorbar()
    plt.title("always bg mask frac")

    plt.figure()
    sz = (self.nr,self.nc,1)
    for j in range(4):
        # choose a frame
        self.frame = self.framessample[j]
        # read frame
        self.im,self.timestamp = self.movie.get_frame(self.frame)
        # read target positions
        self.trxcurr = self.trx.get_frame(self.frame)
        # create isfore mask
        self.isfore_mask()
        # detect objects
        self.obj_detect()
        [y,x] = num.where(self.isobj)
        plt.subplot(1,4,j+1)
        # red = image
        # green = 0
        # blue = isfore
        tmp = num.concatenate((self.im.reshape(sz),num.zeros(sz,dtype=num.uint8),255*self.isfore.astype(num.uint8).reshape(sz)),2)
        plt.imshow(tmp)
        plt.hold('on')
        plt.plot(x,y,'g.')
        plt.title("frame %d"%self.frame)
        plt.axis('image')


    plt.figure()
    plt.plot(self.obj_detection_dist_centers,self.obj_detection_dist_frac_bg,'k.-',
             self.obj_detection_dist_centers,self.obj_detection_dist_frac_fg,'r.-')
    plt.legend(('bg','fg'))
    plt.xlabel('Dist to obj detection')
    plt.ylabel('P(dist | label)')

    plt.show()

    return self

def test2():
    """
    Apply LoG filtering to a sample frame and show the results
    """

    movienames = ['/media/data/data3/pGAWBCyO-GAL4_TrpA_Rig1Plate01Bowl1_20100820T140408/movie.fmf',]
    annnames = ['/media/data/data3/pGAWBCyO-GAL4_TrpA_Rig1Plate01Bowl1_20100820T140408/movie.fmf.ann',]

    self = ExpBGFGModel(movienames,annnames)

    # open movie
    i = 0
    self.movie = movies.Movie( self.movienames[i], params.interactive )
    
    # open annotation
    self.trx = annot.AnnotationFile( self.annnames[i], doreadtrx=True )

    self.frame = 1
    self.im,self.timestamp = self.movie.get_frame(self.frame)
    self.trxcurr = self.trx.get_frame(self.frame)
    self.LoGfilter()

    plt.subplot(131)
    plt.imshow(self.LoG_fil_out)
    plt.title('LoG filter')
    plt.colorbar()
    
    plt.subplot(132)
    plt.imshow(self.LoG_fil_out>=self.LoG_thresh)
    plt.title('LoG filter >= thresh')
    plt.colorbar()
    
    plt.subplot(133)
    plt.imshow(self.im,cmap=cm.gray,vmin=0,vmax=255)
    r,c = num.where(self.isobj)
    plt.plot(c,r,'r.')
    plt.axis('image')
    plt.title('isobj')
    plt.show()
    
    return self

def main():
    """
    main()

    Compute experiment bg and fg models for a set of movies set
    at the start of the function, and plot the results. 
    """

    shortopts = "f:p:m:a:o:s:"
    longopts = ["filename=","params=","movie=","ann=","output=","mat="]
    try:
        opts, args = getopt.getopt(sys.argv[1:], shortopts, longopts)
    except getopt.GetoptError, err:
        # print help information and exit:
        print str(err) # will print something like "option -a not recognized"
        print "ExpBGFGModel options:"
        print "    -f,--filename <name of file containing list of experiment directories>"
        print "      Default: expdirs.txt"
        print "    -p,--params <name of file containing parameters>"
        print "      Default: <empty string>"
        print "    -m,--movie <name of movie file within experiment directories>"
        print "      Default: movie.ufmf"
        print "    -a,--ann <name of annotation file within experiment directories>"
        print "      Default: movie.ufmf.ann"
        print "    -o,--output <name of file to output results to>"
        print "      Default: ExpBGFGModelResults.pickle"
        print "    --mat <name of mat file to output results to>"
        print "      Default: ExpBGFGModelResults.mat"
        sys.exit(2)
        
    expdirsFileName = 'expdirs.txt'
    paramsFileName = ''
    movieFileStr = 'movie.ufmf'
    annFileStr = 'movie.ufmf.ann'
    outputFileName = 'ExpBGFGModelResults.pickle'
    matFileName = 'ExpBGFGModelResults.mat'
    
    for o,a in opts:
        if o in ("-f","--filename"):
            expdirsFileName = a
            print "expdirsFileName = " + a
        elif o in ("-p","--params"):
            paramsFileName = a
            print "paramsFileName = " + a
        elif o in ("-m","--movie"):
            movieFileStr = a
            print "movieFileStr = " + a
        elif o in ("-a","--ann"):
            annFileStr = a
            print "annFileStr = " + a
        elif o in ("-o","--output"):
            outputFileName = a
            print "outputFileName = " + a
        elif o in ("-s","--mat"):
            matFileName = a
            print "matFileName = " + a
        else:
            assert False, "unhandled option"

    # read in the experiment directories
    fid = open(expdirsFileName,"r")
    expdirs = []
    movienames = []
    annnames = []
    for l in fid:
        expdir = l.strip()
        if not os.path.exists(expdir):
            print "Experiment directory %s does not exist. Skipping."%expdir
            continue
        moviename = os.path.join(expdir,movieFileStr)
        if not os.path.exists(moviename):
            print "Movie %s does not exist. Skipping experiment %s."%(moviename,expdir)
            continue
        annname = os.path.join(expdir,annFileStr)
        if not os.path.exists(annname):
            print "Annotation file %s does not exist. Skipping experiment %s."%(annname,expdir)
            continue
        expdirs.append(expdir)
        movienames.append(moviename)
        annnames.append(annname)
    fid.close()
        
    # default parameters
    LoG_hsize=21 
    LoG_sigma=5.
    LoG_nonmaxsup_sz=5 
    LoG_thresh=.5
    difftype='darkfglightbg'
    obj_detection_dist_nbins=100
    isback_dilation_hsize=10
    fg_min_sigma = 1.
    bg_min_sigma = 1.
    min_always_bg_mask_frac = 1. 
    min_frac_cc_covered = .01
    min_area_cc_covered = 1
    backsub_n_bg_std_thresh = 10.
    backsub_n_bg_std_thresh_low = 1.
    backsub_do_use_morphology = False
    backsub_opening_radius = 0
    backsub_closing_radius = 0

    if paramsFileName != '':
        # parse parameters file
        fid = open(paramsFileName,"r")
    
        for l in fid:
            l = l.strip()
            if l[0] == '#':
                continue
            l = l.split('=',1)
            if len(l) < 2:
                print "Skipping parameter line '%s'"%l
                continue
            l[0] = l[0].strip()
            l[1] = l[1].strip()
            if l[0] == 'LoG_hsize':
                LoG_hsize = float(l[1])
            elif l[0] == 'LoG_sigma':
                LoG_sigma = float(l[1])
            elif l[0] == 'LoG_nonmaxsup_sz':
                LoG_nonmaxsup_sz = float(l[1]) 
            elif l[0] == 'LoG_thresh':
                LoG_thresh = float(l[1])
            elif l[0] == 'difftype':
                difftype = l[1]
            elif l[0] == 'obj_detection_dist_nbins':
                obj_detection_dist_nbins = float(l[1])
            elif l[0] == 'isback_dilation_hsize':
                isback_dilation_hsize = float(l[1])
            elif l[0] == 'fg_min_sigma':
                fg_min_sigma = float(l[1])
            elif l[0] == 'bg_min_sigma':
                bg_min_sigma = float(l[1])
            elif l[0] == 'min_always_bg_mask_frac':
                min_always_bg_mask_frac = float(l[1]) 
            elif l[0] == 'min_frac_cc_covered':
                min_frac_cc_covered = float(l[1])
            elif l[0] == 'min_area_cc_covered':
                min_area_cc_covered = float(l[1])
            elif l[0] == 'backsub_n_bg_std_thresh':
                backsub_n_bg_std_thresh = float(l[1])
            elif l[0] == 'backsub_n_bg_std_thresh_low':
                backsub_n_bg_std_thresh_low = float(l[1])
            elif l[0] == 'backsub_do_use_morphology':
                backsub_do_use_morphology = float(l[1]) == 1
            elif l[0] == 'backsub_opening_radius':
                backsub_opening_radius = int(float(l[1]))
            elif l[0] == 'backsub_closing_radius':
                backsub_closing_radius = int(float(l[1]))
            elif l[0] == 'prior_nframessample':
                params.prior_nframessample = float(l[1])
            elif l[0] == 'prior_fg_nsamples_pool':
                params.prior_fg_nsamples_pool = float(l[1])
            elif l[0] == 'prior_bg_nsamples_pool':
                params.prior_bg_nsamples_pool = float(l[1])
            elif l[0] == 'prior_fg_pool_radius_factor':
                params.prior_fg_pool_radius_factor = float(l[1])
            elif l[0] == 'prior_bg_pool_radius_factor':
                params.prior_bg_pool_radius_factor = float(l[1])
        fid.close()

    model = ExpBGFGModel(movienames,annnames,
                        LoG_hsize=LoG_hsize, 
                        LoG_sigma=LoG_sigma,
                        LoG_nonmaxsup_sz=LoG_nonmaxsup_sz, 
                        LoG_thresh=LoG_thresh,
                        difftype=difftype,
                        obj_detection_dist_nbins=obj_detection_dist_nbins,
                        isback_dilation_hsize=isback_dilation_hsize,
                        fg_min_sigma=fg_min_sigma,
                        bg_min_sigma=bg_min_sigma,
                        min_always_bg_mask_frac=min_always_bg_mask_frac,
                        min_frac_cc_covered = min_frac_cc_covered,
                        min_area_cc_covered = min_area_cc_covered,
                        backsub_n_bg_std_thresh = backsub_n_bg_std_thresh,
                        backsub_n_bg_std_thresh_low = backsub_n_bg_std_thresh_low,
                        backsub_do_use_morphology = backsub_do_use_morphology,
                        backsub_opening_radius = backsub_opening_radius,
                        backsub_closing_radius = backsub_closing_radius
                        )
    
    model.est_marginals()
    model.save(outputFileName)
    model.save_mat(matFileName)
    model.show()

    return model
    
if __name__ == "__main__":
    params.interactive = False
    main()



# draw.py
# KMB 11/10/08

import os
import pdb
import sys

import matplotlib
#matplotlib.use( 'WXAgg' )
matplotlib.interactive( True )
import matplotlib.backends.backend_wxagg
import matplotlib.figure
import matplotlib.cm
import matplotlib.pyplot as plt
if hasattr( plt, 'tight_layout' ): # matplotlib >= 1.1
    HAS_TIGHTLAYOUT = True
else:
    HAS_TIGHTLAYOUT = False
import matplotlib.transforms as mtransforms
import numpy as num
import wx
from wx import xrc

import motmot.wxvalidatedtext.wxvalidatedtext as wxvt # part of Motmot

from params import params


#######################################################################
# PlotConstants
#######################################################################
class PlotConstants:
    def __init__( self ):
        self.vel_bins = 100
        self.vel_x_min=self.vel_x_max=self.vel_y_min=self.vel_y_max = 'a'
        self.dth_bins = 100
        self.dth_x_min=self.dth_x_max=self.dth_y_min=self.dth_y_max = 'a'
##        self.orn_bins = 90
##        self.orn_x_min=self.orn_x_max=self.orn_y_min=self.orn_y_max = 'a'
##        self.space_bins = 50
##        self.space_x_min=self.space_x_max=self.space_y_min=self.space_y_max = 'a'
        self.pos_binsize = 50
        self.pos_x_min=self.pos_x_max=self.pos_y_min=self.pos_y_max = 'a'

        self.filepath = None
const = PlotConstants()


#######################################################################
# PlotPanel
#######################################################################
# class from http://www.scipy.org/Matplotlib_figure_in_a_wx_panel
# although menu/right-click/save functionality was added here
class PlotPanel(wx.Panel):
    """The PlotPanel has a Figure and a Canvas. OnSize events simply set a 
    flag, and the actually redrawing of the figure is triggered by an Idle event."""
    ###################################################################
    # __init__()
    ###################################################################
    def __init__(self, parent, id=wx.ID_ANY, color=None,
                 dpi=None, style=wx.NO_FULL_REPAINT_ON_RESIZE, **kwargs):
        wx.Panel.__init__( self, parent, id=id, style=style, **kwargs )
        self.figure = matplotlib.figure.Figure( None, dpi )
        self.canvas = matplotlib.backends.backend_wxagg.FigureCanvasWxAgg( self, -1, self.figure )
        self.SetColor( color )
        self.Bind( wx.EVT_IDLE, self._onIdle )
        self.Bind( wx.EVT_SIZE, self._onSize )
        self._resizeflag = True
        self._SetSize()

        self.canvas.Bind( wx.EVT_RIGHT_UP, self.OnRightMouseButton )

        self.canvas.mpl_connect( 'draw_event', self.post_draw )
            

    ###################################################################
    # OnRightMouseButton()
    ###################################################################
    def OnRightMouseButton( self, evt ):
        """Right mouse button pressed; pop up save menu."""
        menu = wx.Menu()
        file_save_item = wx.MenuItem( menu, wx.ID_ANY, "Save as..." )
        menu.AppendItem( file_save_item )
        self.canvas.Bind( wx.EVT_MENU, self.OnMenuSave )
        self.canvas.PopupMenu( menu )


    ###################################################################
    # OnMenuSave()
    ###################################################################
    def OnMenuSave( self, evt ):
        """User has chosen to save this figure as an image file.
        Prompt for filename and save."""
        # the extension on filename determines file format
        # create text list of allowed file extensions
        extensions = {'.eps': 'encapsulated postscript (*.eps)',
                      '.png': 'portable network graphics (*.png)',
                      '.pdf': 'portable data format (*.pdf)'}
        dialog_str = ''
        ext_list = []
        for ext, txt in extensions.iteritems():
            dialog_str += txt + '|*' + ext + '|'
            ext_list.append( ext )
        dialog_str = dialog_str[:-1]

        # make default filename
        (dirname, filename) = os.path.split( const.filepath )
        (basename, ext) = os.path.splitext( filename )
        if self.__class__.__name__ == 'TrajectoryPlotPanel':
            if '_trajplot' not in basename:
                basename += '_trajplot'
        elif self.__class__.__name__ == 'PosHistPanel':
            if '_poshist' not in basename:
                basename += '_poshist'
        elif self.__class__.__name__ == 'VelocityPlotPanel':
            if '_velocityplot' not in basename:
                basename += '_velocityplot'
        elif self.__class__.__name__ == 'VelPlotPanel':
            if '_velhist' not in basename:
                basename += '_velhist'
        elif self.__class__.__name__ == 'TurnPlotPanel':
            if '_turnhist' not in basename:
                basename += '_turnhist'
        elif self.__class__.__name__ == 'SpacePlotPanel':
            if '_spaceplot' not in basename:
                basename += '_spaceplot'
        elif self.__class__.__name__ == 'OrnPlotPanel':
            if '_ornplot' not in basename:
                basename += '_ornplot'

        # show dialog
        dlg = wx.FileDialog( self.canvas, "Save as image...", dirname, basename, wildcard=dialog_str, style=wx.SAVE )
        if dlg.ShowModal() == wx.ID_OK:
            # get entered filename and extension
            const.filepath = dlg.GetPath()
            selected_ext = ext_list[dlg.GetFilterIndex()]
            if not const.filepath.endswith( selected_ext ):
                const.filepath += selected_ext
            print "saving to", const.filepath
            try:
                self.figure.savefig( const.filepath )
            except IOError:
                wx.MessageBox( "Miscellaneous error while saving.",
                               "Write Error", wx.ICON_ERROR|wx.OK )
                # everybody loves a "miscellaneous error" message

        dlg.Destroy()


    ###################################################################
    # SetColor()
    ###################################################################
    def SetColor(self, rgbtuple):
        """Set figure and canvas colours to be the same."""
        if not rgbtuple:
            rgbtuple = wx.SystemSettings.GetColour(wx.SYS_COLOUR_BTNFACE).Get()
        col = [c/255.0 for c in rgbtuple]
        self.figure.set_facecolor(col)
        self.figure.set_edgecolor(col)
        self.canvas.SetBackgroundColour(wx.Colour(*rgbtuple))


    ###################################################################
    # _onSize()
    ###################################################################
    def _onSize(self, event):
        self._resizeflag = True


    ###################################################################
    # _onIdle()
    ###################################################################
    def _onIdle(self, evt):
        if self._resizeflag:
            self._resizeflag = False
            self._SetSize()
            self.draw()


    ###################################################################
    # _SetSize()
    ###################################################################
    def _SetSize(self, pixels=None):
        """This method can be called to force the Plot to be a desired size,
        which defaults to the ClientSize of the panel."""

        if not pixels:
            pixels = self.GetClientSize()
        self.canvas.SetSize(pixels)
        self.figure.set_size_inches(pixels[0]/self.figure.get_dpi(),
                                    pixels[1]/self.figure.get_dpi())


    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        """Drawing events handled here. Create subplots, label axes, plot."""
        pass
    

    ###################################################################
    # post_draw()
    ###################################################################
    def post_draw( self, evt ):
        """Post-drawing events handled here. Rescale plots to fit in panel."""
        if HAS_TIGHTLAYOUT and False:
            plt.tight_layout()
            
        else:
            # http://matplotlib.sourceforge.net/faq/howto_faq.html#automatically-make-room-for-tick-labels
            # http://stackoverflow.com/questions/4018860/text-box-in-matplotlib
            bboxes = []
            # Cycle through all artists in all the axes in the figure
            for ax in self.figure.axes:
                for artist in ax.get_children():
                    # If it's a text artist, grab its bounding box...
                    if isinstance(artist, matplotlib.text.Text):
                        bbox = artist.get_window_extent()

                        # the figure transform goes from relative->pixels and
                        # we want the inverse of that
                        bboxi = bbox.inverse_transformed(self.figure.transFigure)
                        bboxes.append(bboxi)
                # cycle through all tick labels, too
                for label in ax.get_xticklabels() + ax.get_yticklabels():
                    bbox = label.get_window_extent()
                    bboxi = bbox.inverse_transformed(self.figure.transFigure)
                    bboxes.append(bboxi)

            # this is the bbox that bounds all the bboxes, again in
            # relative figure coords
            if len( bboxes ) == 0: return
            size = mtransforms.Bbox.union(bboxes)
            margins = self.figure.subplotpars

            vspace = size.height - (margins.top - margins.bottom)
            if size.height > 0.8:
                self.figure.subplots_adjust( bottom=margins.bottom + vspace/2.,
                                             top=margins.top - vspace/2. )
            elif size.height < 0.5:
                self.figure.subplots_adjust( bottom=margins.bottom - vspace/2.,
                                             top=margins.top + vspace/2. )
            hspace = size.width - (margins.right - margins.left)
            if size.width > 0.8:
                self.figure.subplots_adjust( left=margins.left + hspace/2.,
                                             right=margins.right - hspace/2. )
            elif size.width < 0.5:
                self.figure.subplots_adjust( left=margins.left - hspace/2.,
                                             right=margins.right + hspace/2. )

            # Temporarily disconnect any callbacks to the draw event...
            # (To avoid recursion)
            func_handles = self.figure.canvas.callbacks.callbacks[evt.name]
            self.figure.canvas.callbacks.callbacks[evt.name] = {}
            # Re-draw the figure..
            self.figure.canvas.draw()
            # Reset the draw event callbacks
            self.figure.canvas.callbacks.callbacks[evt.name] = func_handles


#######################################################################
# TrajectoryPlotPanel
#######################################################################
class TrajectoryPlotPanel (PlotPanel):
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, parent, data ):
        """Initialize a trajectory plot panel with data."""
        PlotPanel.__init__( self, parent )

        self.x = num.zeros( (len( data ),max( len( data[0] ), params.nids )), dtype=num.float32 )
        self.y = num.zeros( self.x.shape, dtype=num.float32 )
        for fi, frame in enumerate( data ):
            for ei, ellipse in frame.iteritems():
                if ei >= self.x.shape[1]: break
                self.x[fi,ei] = ellipse.center.x
                self.y[fi,ei] = ellipse.center.y
        
        
    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        """Draw the data."""
        if not hasattr( self, 'subplot' ):
            self.subplot = self.figure.add_subplot( 111, aspect='equal' )
            self.subplot.set_title( "Positions in first %d frames"%self.x.shape[0], fontsize=12 )
            self.subplot.set_xlabel( "x (pixels)" )
            self.subplot.set_ylabel( "y (pixels)" )

        self.subplot.plot( self.x, self.y )


#######################################################################
# VelocityPlotPanel
#######################################################################
class VelocityPlotPanel (PlotPanel):
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, parent, data ):
        """Initialize a velocity plot panel with data."""
        PlotPanel.__init__( self, parent )

        self.x = num.zeros( (len( data ) - 1,), dtype=num.float32 )
        self.y = num.zeros( (len( data ) - 1,max( len( data[0] ), params.nids )), dtype=num.float32 )
        for fi in range( len( data ) - 1 ):
            self.x[fi] = fi
            for ei in range( self.y.shape[1] ):
                if data[fi].hasItem( ei ) and data[fi + 1].hasItem( ei ):
                    pos = data[fi][ei]
                    next_pos = data[fi + 1][ei]
                    self.y[fi,ei] = pos.Euc_dist( next_pos )

        try:
            self.y_mn = num.mean( self.y, 1 )
            self.y_st = num.std( self.y, 1 )
        except FloatingPointError:
            self.y_mn = self.x
            self.y_st = self.x
        #self.y_env = num.hstack( (self.y_mn + self.y_st,
        #                          self.y_mn[::-1] - self.y_st[::-1]) )
        #self.x_env = num.hstack( (self.x, self.x[::-1]) )
        
        
    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        """Draw the data."""
        if not hasattr( self, 'subplot' ):
            self.subplot = self.figure.add_subplot( 111 )
            self.subplot.set_title( "Velocities in first %d frames"%self.x.shape[0], fontsize=12 )
            self.subplot.set_xlabel( "frame" )
            self.subplot.set_ylabel( "velocity (pixels/frame)\nmean +/- S.D." )

        self.subplot.plot( self.x, self.y_mn, 'k' )
        self.subplot.fill_between( self.x, self.y_mn + self.y_st, self.y_mn - self.y_st, alpha=0.5, edgecolor='none', facecolor='k' )
        if self.y.shape[1] < 10:
            try:
                self.subplot.plot( self.x, self.y )
            except ZeroDivisionError:
                pass
        

#######################################################################
# PosHistPanel
#######################################################################
class PosHistPanel( PlotPanel ):
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, frame, data, width, height ):
        PlotPanel.__init__( self, frame )
        self.data = data

        # fill grid
        self.grid = num.zeros( (width/const.pos_binsize+1, height/const.pos_binsize+1) )
        for frame in data:
            for fly in frame.itervalues():
                fly_x = fly.center.x
                fly_y = fly.center.y
                try: self.grid[int(fly_x/const.pos_binsize),int(fly_y/const.pos_binsize)] += 1
                except IndexError:
                    print "error adding", fly_x, fly_y, "to bin", fly_x/const.pos_binsize, fly_y/const.pos_binsize, "(size is", self.grid.shape, "\b)"
                # increment grid
            # for each fly
        # for each frame

        self.xticks = range( 0, self.grid.shape[0], max( 200/const.pos_binsize, 1 ) )
        self.xticklabels = []
        for xx in self.xticks:
            self.xticklabels.append( str(xx*const.pos_binsize) )
        self.yticks = range( 0, self.grid.shape[1], max( 200/const.pos_binsize, 1 ) )
        self.yticklabels = []
        for yy in self.yticks:
            self.yticklabels.append( str(yy*const.pos_binsize) )


    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        first = False
        if not hasattr(self, 'subplot'):
            self.subplot = self.figure.add_subplot( 111, aspect='equal' )
            self.subplot.set_title("Position histogram for first %d frames"%len( self.data ), fontsize = 12)
            self.subplot.set_xlabel( "x (pixels)" )
            self.subplot.set_ylabel( "y (pixels)" )
            self.subplot.set_xticks( self.xticks )
            self.subplot.set_yticks( self.yticks )
            self.subplot.set_xticklabels( self.xticklabels )
            self.subplot.set_yticklabels( self.yticklabels )
            first = True

        # 2D histogram
        self.subplot.imshow( self.grid.T, cmap=matplotlib.cm.hot )

        if first:
            self.subplot.invert_yaxis()


#######################################################################
# VelPlotPanel
#######################################################################
class VelPlotPanel( PlotPanel ):
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, frame, data ):
        PlotPanel.__init__( self, frame )
        self.data = data

        self.vels = []
        for ff in range( len(self.data)-1 ):
            for fly in self.data[ff].iterkeys():
                if self.data[ff+1].hasItem(fly):
                    vel = self.data[ff][fly].Euc_dist( self.data[ff+1][fly] )
                    if True:#vel < max_jump:
                        self.vels.append( vel )
                # calculate velocity
            # for each fly
        # for each pair of consecutive frames in data
        self.vels = num.array( self.vels )


    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        if not hasattr(self, 'subplot'):
            self.subplot = self.figure.add_subplot(111)
            self.subplot.set_title("Velocity histogram for first %d frames"%len( self.data ), fontsize = 12)
            self.subplot.set_xlabel( "fly velocity (pixels/frame)" )
            self.subplot.set_ylabel( "bin count" )

        # histogram
        self.subplot.hist( self.vels, const.vel_bins )
        
        if const.vel_x_min != 'a':
            self.subplot.axis( xmin=const.vel_x_min )
        if const.vel_x_max != 'a':
            self.subplot.axis( xmax=const.vel_x_max )
        if const.vel_y_min != 'a':
            self.subplot.axis( ymin=const.vel_y_min )
        if const.vel_y_max != 'a':
            self.subplot.axis( ymax=const.vel_y_max )


#######################################################################
# TurnPlotPanel
#######################################################################
class TurnPlotPanel( PlotPanel ):
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, frame, data ):
        PlotPanel.__init__( self, frame )
        self.data = data

        self.turns = []
        for ff in range( len(self.data)-1 ):
            for fly in self.data[ff].iterkeys():
                if self.data[ff+1].hasItem(fly):
                    turn = self.data[ff][fly].angle - self.data[ff+1][fly].angle
                    if abs( turn ) < num.pi:
                        self.turns.append( turn )
                # calculate dtheta
            # for each fly
        # for each pair of consecutive frames in data
        self.turns = num.array( self.turns )


    ###################################################################
    # draw()
    ###################################################################
    def draw( self ):
        if not hasattr(self, 'subplot'):
            self.subplot = self.figure.add_subplot(111)
            self.subplot.set_title("Turn histogram for first %d frames"%len( self.data ), fontsize = 12)
            self.subplot.set_xlabel( "turn velocity (rad/frame)" )
            self.subplot.set_ylabel( "bin count" )

        # histogram
        self.subplot.hist( self.turns, const.dth_bins )
        self.subplot.axis( xmin=-num.pi, xmax=num.pi )
        
        if const.dth_x_min != 'a':
            self.subplot.axis( xmin=const.dth_x_min )
        if const.dth_x_max != 'a':
            self.subplot.axis( xmax=const.dth_x_max )
        if const.dth_y_min != 'a':
            self.subplot.axis( ymin=const.dth_y_min )
        if const.dth_y_max != 'a':
            self.subplot.axis( ymax=const.dth_y_max )


class OrnPlotPanel( PlotPanel ):
    def __init__( self, frame, data, title ):
        PlotPanel.__init__( self, frame )
        self.data = data
        self.title = title

        self.orns = []
        for frame in self.data:
            for fly in frame.itervalues():
                self.orns.append( fly.angle )
                # grab angle
            # for each ellipse
        # for each frame in data
        self.orns = num.array( self.orns )
        
    def draw( self ):
        if not hasattr(self, 'subplot'):
            self.subplot = self.figure.add_subplot(111)

        # histogram
        self.subplot.hist( self.orns, const.orn_bins )
        
        #Set some plot attributes
        self.subplot.set_title(self.title, fontsize = 12)
        self.subplot.set_xlabel( "fly orientation (rad)" )
        self.subplot.set_ylabel( "bin count" )
        
class SpacePlotPanel( PlotPanel ):
    def __init__( self, frame, data, title ):
        PlotPanel.__init__( self, frame )
        self.data = data
        self.title = title
        
        self.dists = []
        for frame in self.data:
            for fly1 in frame.iterkeys():
                for fly2 in frame.iterkeys():
                    if fly1 < fly2:
                        self.dists.append( frame[fly1].Euc_dist( frame[fly2] ) )
                    # calculate distance
                # for each pair of ellipses
        # for each frame in data
        self.dists = num.array( self.dists )

    def draw( self ):
        if not hasattr(self, 'subplot'):
            self.subplot = self.figure.add_subplot(111)

        # histogram
        self.subplot.hist( self.dists, const.space_bins )
        
        #Set some plot attributes
        self.subplot.set_title(self.title, fontsize = 12)
        self.subplot.set_xlabel( "distance between flies (pixels)" )
        self.subplot.set_ylabel( "bin count" )
        

class DrawSettingsDialog:
    def __init__( self, parent=None ):
        rsrc = xrc.XmlResource( RSRC_FILE )
        self.frame = rsrc.LoadFrame( parent, "frame_Ctrax_settings" )

        # input box handles
        self.vel_bins_box = xrc.XRCCTRL( self.frame, "text_vel_bins" )
        self.orn_bins_box = xrc.XRCCTRL( self.frame, "text_orn_bins" )
        self.space_bins_box = xrc.XRCCTRL( self.frame, "text_space_bins" )
        self.pos_binsize_box = xrc.XRCCTRL( self.frame, "text_pos_binsize" )
        self.vel_axes_box = xrc.XRCCTRL( self.frame, "text_vel_axes" )
        self.orn_axes_box = xrc.XRCCTRL( self.frame, "text_orn_axes" )
        self.space_axes_box = xrc.XRCCTRL( self.frame, "text_space_axes" )
        self.pos_axes_box = xrc.XRCCTRL( self.frame, "text_pos_axes" )

        # initial values
        self.vel_bins_box.SetValue( str(const.vel_bins) )
        self.orn_bins_box.SetValue( str(const.orn_bins) )
        self.space_bins_box.SetValue( str(const.space_bins) )
        self.pos_binsize_box.SetValue( str(const.pos_binsize) )
        self.vel_axes_box.SetValue( "a,a,a,a" )
        self.orn_axes_box.SetValue( "a,a,a,a" )
        self.space_axes_box.SetValue( "a,a,a,a" )
        self.pos_axes_box.SetValue( "a,a,a,a" )

        # bind to wxvalidatedtext
        wxvt.setup_validated_integer_callback( self.vel_bins_box,
                                               xrc.XRCID("text_vel_bins"),
                                               self.OnTextValidated,
                                               pending_color=params.wxvt_bg )
        wxvt.setup_validated_integer_callback( self.orn_bins_box,
                                               xrc.XRCID("text_orn_bins"),
                                               self.OnTextValidated,
                                               pending_color=params.wxvt_bg )
        wxvt.setup_validated_integer_callback( self.space_bins_box,
                                               xrc.XRCID("text_space_bins"),
                                               self.OnTextValidated,
                                               pending_color=params.wxvt_bg )
        wxvt.setup_validated_integer_callback( self.pos_binsize_box,
                                               xrc.XRCID("text_pos_binsize"),
                                               self.OnTextValidated,
                                               pending_color=params.wxvt_bg )
        wxvt.Validator( self.vel_axes_box, xrc.XRCID("text_vel_axes"),
                        self.OnTextValidated, self.ParseAxes,
                        pending_color=params.wxvt_bg )
        wxvt.Validator( self.orn_axes_box, xrc.XRCID("text_orn_axes"),
                        self.OnTextValidated, self.ParseAxes,
                        pending_color=params.wxvt_bg )
        wxvt.Validator( self.space_axes_box, xrc.XRCID("text_space_axes"),
                        self.OnTextValidated, self.ParseAxes,
                        pending_color=params.wxvt_bg )
        wxvt.Validator( self.pos_axes_box, xrc.XRCID("text_pos_axes"),
                        self.OnTextValidated, self.ParseAxes,
                        pending_color=params.wxvt_bg )

        self.frame.Show()

    def OnTextValidated( self, evt ):
        """User-entered text has been validated; set global variables
        correspondingly."""
        if evt.GetId() == xrc.XRCID( "text_vel_bins" ):
            const.vel_bins = int(self.vel_bins_box.GetValue())
        elif evt.GetId() == xrc.XRCID( "text_orn_bins" ):
            const.orn_bins = int(self.orn_bins_box.GetValue())
        elif evt.GetId() == xrc.XRCID( "text_space_bins" ):
            const.space_bins = int(self.space_bins_box.GetValue())
        elif evt.GetId() == xrc.XRCID( "text_pos_binsize" ):
            const.pos_binsize = int(self.pos_binsize_box.GetValue())
        elif evt.GetId() == xrc.XRCID( "text_vel_axes" ):
            const.vel_x_min = self.new_axes_vals[0]
            const.vel_x_max = self.new_axes_vals[1]
            const.vel_y_min = self.new_axes_vals[2]
            const.vel_y_max = self.new_axes_vals[3]
            # set axes box values to currently stored values (removes user formatting)
            self.vel_axes_box.SetValue( "%s,%s,%s,%s"%(str(const.vel_x_min),
                                                       str(const.vel_x_max),
                                                       str(const.vel_y_min),
                                                       str(const.vel_y_max)) )
        elif evt.GetId() == xrc.XRCID( "text_orn_axes" ):
            const.orn_x_min = self.new_axes_vals[0]
            const.orn_x_max = self.new_axes_vals[1]
            const.orn_y_min = self.new_axes_vals[2]
            const.orn_y_max = self.new_axes_vals[3]
            # set axes box values to currently stored values (removes user formatting)
            self.orn_axes_box.SetValue( "%s,%s,%s,%s"%(str(const.orn_x_min),
                                                       str(const.orn_x_max),
                                                       str(const.orn_y_min),
                                                       str(const.orn_y_max)) )
        elif evt.GetId() == xrc.XRCID( "text_space_axes" ):
            const.space_x_min = self.new_axes_vals[0]
            const.space_x_max = self.new_axes_vals[1]
            const.space_y_min = self.new_axes_vals[2]
            const.space_y_max = self.new_axes_vals[3]
            # set axes box values to currently stored values (removes user formatting)
            self.space_axes_box.SetValue( "%s,%s,%s,%s"%(str(const.space_x_min),
                                                       str(const.space_x_max),
                                                       str(const.space_y_min),
                                                       str(const.space_y_max)) )
        elif evt.GetId() == xrc.XRCID( "text_pos_axes" ):
            const.pos_x_min = self.new_axes_vals[0]
            const.pos_x_max = self.new_axes_vals[1]
            const.pos_y_min = self.new_axes_vals[2]
            const.pos_y_max = self.new_axes_vals[3]
            # set axes box values to currently stored values (removes user formatting)
            self.pos_axes_box.SetValue( "%s,%s,%s,%s"%(str(const.pos_x_min),
                                                       str(const.pos_x_max),
                                                       str(const.pos_y_min),
                                                       str(const.pos_y_max)) )

    def ParseAxes( self, string ):
        """Parse user input for drawings axes limits. Return True or False
        depending on whether input is valid."""
        vals = string.split( ',' ) # split on commas
        if len(vals) != 4: vals = string.split() # try again with whitespace
        if len(vals) != 4:
            return False

        # strip parentheses, etc. and make numeric
        for vv in range( len(vals) ):
            vals[vv] = vals[vv].strip( '()[], ' )
            # test for empty
            if vals[vv] == '': return False
            # test for a
            if vals[vv] == 'a': continue
            # test for int
            if vals[vv].isdigit() or \
                   (vals[vv][0] == '-' and vals[vv][1:].isdigit()): # negative int
                vals[vv] = int(vals[vv])
                continue
            # test for float
            try:
                vals[vv] = float(vals[vv])
            except ValueError: pass
            else: continue

            return False

        # validation successful; save values so we don't have to re-parse later
        self.new_axes_vals = vals
        return True



#######################################################################
# DebugPlotPanel
#######################################################################
class DebugPlotPanel( PlotPanel ):
    """Show an image in a matplotlib window."""
    ###################################################################
    # imshow()
    ###################################################################
    def imshow(self,im):
        self.grid = im
        print self.grid.shape
        print self.grid.T


    ###################################################################
    # draw()
    ###################################################################
    def draw(self):
        if not hasattr( self, 'subplot' ):
            self.subplot = self.figure.add_subplot( 111 )
        if hasattr(self,'grid'):
            self.subplot.imshow( self.grid, cmap=matplotlib.cm.jet )



# tracking_wizard.py
# JAB 10/25/11

import os.path
import random
import sys
from threading import Timer
import time

import numpy as num
import wx
from wx import xrc

import bg_settings
from codedir import codedir
from ellipsesk import est_shape
from fixbg import FixBG
if 'darwin' in sys.platform:
    from mac_text_fixer import fix_text_sizes
from params import params
from roi import ROI
from setarena import SetArena
from tracking_settings import TrackingSettings
from zoomable_image_canvas import ZoomableImageCanvas

RSRC_FILE = os.path.join( codedir, 'xrc', 'tracking_wizard.xrc')

#######################################################################
# TrackingWizard
#######################################################################
class TrackingWizard:
    ###################################################################
    # __init__()
    ###################################################################
    def __init__( self, parent_frame, bg, n_frames,
                  get_frame_callback,
                  recalculate_bg_callback,
                  starttracking_callback,
                  isSBFMF ):
        """Initialize frame and bind callbacks. Caller must Show() frame."""
        self.bg = bg # bg.BackgroundCalculator object
        self.n_frames = n_frames # number of frames in movie
        self.isSBFMF = isSBFMF

        # make frame
        rsrc = xrc.XmlResource( RSRC_FILE )
        self.frame = rsrc.LoadFrame( parent_frame, "frame_wizard" )

        if 'darwin' in sys.platform:
            fix_text_sizes( self.frame )

        # connect variables
        self.reset_button = xrc.XRCCTRL( self.frame, "button_reset_frames" )
        self.prev_button = xrc.XRCCTRL( self.frame, "button_previous" )
        self.default_prev_label = self.prev_button.GetLabel()
        self.next_button = xrc.XRCCTRL( self.frame, "button_next" )
        self.default_next_label = self.next_button.GetLabel()
        self.title_text = xrc.XRCCTRL( self.frame, "text_title" )
        self.instruction_text = xrc.XRCCTRL( self.frame, "text_instructions" )

        self.control_panel = xrc.XRCCTRL( self.frame, "panel_control" )
        self.parent_panel = xrc.XRCCTRL( self.frame, "panel_parent" )
        self.single_panel = xrc.XRCCTRL( self.frame, "panel_single" )
        self.multi_panel = xrc.XRCCTRL( self.frame, "panel_multiple" )

        # bind callbacks
        self.frame.Bind( wx.EVT_SIZE, self.OnResize )
        self.frame.Bind( wx.EVT_BUTTON, self.OnResetFrames, id=xrc.XRCID("button_reset_frames") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnPrevButton, id=xrc.XRCID("button_previous") )
        self.frame.Bind( wx.EVT_BUTTON, self.OnNextButton, id=xrc.XRCID("button_next") )

        # remember parent's callbacks
        self.get_frame_callback = get_frame_callback
        self.recalculate_bg_callback = recalculate_bg_callback
        self.starttracking_callback = starttracking_callback

        # list panels
        self.img_winds = []
        for i in range( 9 ):
            panel = xrc.XRCCTRL( self.frame, "panel%d"%i )
            stext = xrc.XRCCTRL( self.frame, "text_fn%d"%i )

            box = wx.BoxSizer( wx.VERTICAL )
            panel.SetSizer( box )
            canvas = ZoomableImageCanvas( panel )
            box.Add( canvas, 1, wx.EXPAND )
            panel.SetAutoLayout( True )
            panel.Layout()

            self.img_winds.append( {'panel': panel,
                                    'stext': stext,
                                    'canvas': canvas,
                                    'framenumber': i,
                                    'n_obs': 0} )
        panel = xrc.XRCCTRL( self.frame, "panel_img" )
        box = wx.BoxSizer( wx.VERTICAL )
        panel.SetSizer( box )
        canvas = ZoomableImageCanvas( panel )
        box.Add( canvas, 1, wx.EXPAND )
        panel.SetAutoLayout( True )
        panel.Layout()
        self.single_wind = {'panel': panel,
                            'stext': None,
                            'canvas': canvas,
                            'framenumber': 0,
                            'n_obs': 0}

        self.prev_button.Enable( False )
        self.prev_calls = []
        self.cur_call = None
        self.next_calls = []

        params.n_bg_std_thresh = 15
        params.n_bg_std_thresh_low = 5
        self.print_n_obs = False
        self.reset_frame_numbers()
        
        self.__button_reset_timer_running = False
        
        self.init_no_bg()


    ###################################################################
    # show_single()
    ###################################################################
    def show_single( self, evt=None ):
        """Show the large, single-frame image panel."""
        self.multi_panel.Hide()
        self.single_panel.Show()
        self.reset_button.Hide()
        self.redraw()
        self.OnResize( evt )


    ###################################################################
    # show_multi()
    ###################################################################
    def show_multi( self, evt=None ):
        """Show the multi-frame image panel."""
        self.single_panel.Hide()
        self.multi_panel.Show()
        self.reset_button.Show()
        self.redraw()
        self.OnResize( evt )


    ###################################################################
    # showing_winds()
    ###################################################################
    def showing_winds( self ):
        """Return the list of which image windows are currently displayed."""
        if self.multi_panel.IsShown():
            return (self.img_winds, self.multi_panel.GetSizer())
        elif self.single_panel.IsShown():
            return ([self.single_wind], self.single_panel.GetSizer())
        return (None, None)
        

    ###################################################################
    # OnResize()
    ###################################################################
    def OnResize( self, evt ):
        """Window was resized. Rescale images."""
        if evt is not None:
            evt.Skip()

        self.single_panel.SetSize( self.parent_panel.GetSize() )
        self.multi_panel.SetSize( self.parent_panel.GetSize() )

        (winds_to_show, sizer) = self.showing_winds()

        if hasattr( self, 'img_size' ) and winds_to_show is not None:
            for wind_data in winds_to_show:
                panel_item = sizer.GetItem( wind_data['panel'], recursive=True )
                panel_item.SetRatio( float( self.img_size[1] )/self.img_size[0] )
                
            self.frame.Layout()

            for wind_data in winds_to_show:
                wind_data['canvas'].redraw()


    ###################################################################
    # redraw()
    ###################################################################
    def redraw( self ):
        """Redraw the images in all the showing image windows."""
        
        (winds_to_show, sizer) = self.showing_winds()
        if winds_to_show is None: return

        prev_enabled = self.prev_button.Enabled
        next_enabled = self.next_button.Enabled
        self.prev_button.Enable( False )
        self.next_button.Enable( False )

        fmt = 'MONO8'
        line_segs = []
        line_clrs = []
        line_widths = []
        img_src = 'img'

        # erase
        if hasattr( self.bg, 'center' ) and hasattr( self, 'img_size' ) and \
               self.showing_image_type != params.SHOW_RAW_FRAME and \
               self.showing_image_type != params.SHOW_BACKGROUND:
            for wind_data in winds_to_show:
                img = num.zeros( self.img_size[:2], dtype=num.uint8 ) + 127
                wind_data['canvas'].update_image_and_drawings( wind_data['canvas'].current_source,
                                                               img,
                                                               format=fmt )
                wind_data['n_obs'] = 0

        # redraw
        for wind_data in winds_to_show:

            if self.showing_image_type == params.SHOW_RAW_FRAME:
                img, stamp = self.get_frame_callback( wind_data['framenumber'] )
                self.img_size = img.shape

            elif hasattr( self.bg, 'center' ):
                initial_high = params.n_bg_std_thresh
                initial_low = params.n_bg_std_thresh_low
                
                img, fmt, line_segs, line_clrs, a,b,c, n_obs, d = self.bg.get_sub_image( self.showing_image_type, wind_data['framenumber'] )
                line_widths = [1 for c in line_clrs]
                self.img_size = img.shape
                
                wind_data['n_obs'] = n_obs

                try:
                    wx.Yield()
                except: pass
                
                if params.n_bg_std_thresh != initial_high or \
                       params.n_bg_std_thresh_low != initial_low:
                    # user changed slider, abort this redraw
                    self.prev_button.Enable( prev_enabled )
                    self.next_button.Enable( next_enabled )
                    return

            else:
                self.img_size = (101,101)
                img = num.zeros( self.img_size, dtype=num.uint8 )
                img_src = 'blank'

            wind_data['canvas'].update_image_and_drawings( img_src, img,
                                                           format=fmt,
                                                           linesegs=line_segs,
                                                           lineseg_colors=line_clrs,
                                                           lineseg_widths=line_widths )
            
        if self.print_n_obs:
            self.reset_framenumber_text()
            
        self.prev_button.Enable( prev_enabled )
        self.next_button.Enable( next_enabled )


    ###################################################################
    # resize_control_panel()
    ###################################################################
    def resize_control_panel( self ):
        """Set size of control panel to fill width of instruction box."""
        w = self.frame.GetSize()[0] - 30 - self.parent_panel.GetSize()[0]
        if sys.platform.startswith( 'win' ):
            w -= 15
        self.control_panel.SetMinSize( (w,self.control_panel.GetSize()[1]) )


    ###################################################################
    # new_control_radio_box()
    ###################################################################
    def new_control_radio_box( self, choices ):
        """Make a new radio box with specified choices, in control panel."""
        self.resize_control_panel()
        box = wx.BoxSizer( wx.VERTICAL )
        self.control_panel.SetSizer( box )

        self.radio_control = wx.RadioBox( self.control_panel, majorDimension=3, style=wx.RA_SPECIFY_ROWS, choices=choices )        
        box.Add( self.radio_control, 1, wx.EXPAND )
        
        self.control_panel.SetAutoLayout( True )
        self.control_panel.Layout()


    ###################################################################
    # get_radio_control_selection()
    ###################################################################
    def get_radio_control_selection( self ):
        """Return the index of the selected choice in the radio control."""
        if hasattr( self, 'radio_control' ):
            sel = self.radio_control.GetSelection()
            self.radio_control.Destroy()
            delattr( self, 'radio_control' )
            return sel
        return None


    ###################################################################
    # new_slider()
    ###################################################################
    def new_slider( self, value, text ):
        """Make a new slider with an initial value and text legend."""
        self.resize_control_panel()
        box = wx.BoxSizer( wx.VERTICAL )
        self.control_panel.SetSizer( box )

        box.Add( (-1,10), 0 )

        if 'darwin' in sys.platform:
            self.slider = wx.Slider( self.control_panel, value=value, minValue=1, maxValue=255 )
            self.slider.SetPageSize( 10 )
            self.slider.GetThumbPosition = self.slider.GetValue
            self.slider.SetThumbPosition = self.slider.SetValue
        else:
            self.slider = wx.ScrollBar( self.control_panel )
            self.slider.SetScrollbar( value, 1, 255, 10 )
        self.slider.SetMinSize( (self.control_panel.GetMinSize()[0],-1) )
        box.Add( self.slider )

        self.slider_text = wx.StaticText( self.control_panel, label=text )
        box.Add( self.slider_text, 0, wx.ALIGN_CENTER_HORIZONTAL )

        self.control_panel.SetAutoLayout( True )
        self.control_panel.Layout()


    ###################################################################
    # remove_slider()
    ###################################################################
    def remove_slider( self ):
        """Remove the slider control."""
        if hasattr( self, 'slider' ):
            self.slider.Destroy()
            delattr( self, 'slider' )
            self.slider_text.Destroy()
            delattr( self, 'slider_text' )


    ###################################################################
    # cleanup_controls()
    ###################################################################
    def cleanup_controls( self ):
        """Remove all controls."""
        self.remove_slider()
        self.get_radio_control_selection()

        self.print_n_obs = False
        self.reset_framenumber_text()


    ###################################################################
    # reset_frame_numbers()
    ###################################################################
    def reset_frame_numbers( self ):
        """Choose new random frame numbers for each image window."""
        framenumbers = num.zeros( (len( self.img_winds ),), dtype=num.int32 ) - 1
        for i in range( len( self.img_winds ) ):
            f = random.randrange( 0, self.n_frames )
            while f in framenumbers:
                f = random.randrange( 0, self.n_frames )
            framenumbers[i] = f
        framenumbers.sort()

        for i in range( len( self.img_winds ) ):
            self.img_winds[i]['framenumber'] = framenumbers[i]

        self.reset_framenumber_text()

        
    ###################################################################
    # reset_framenumber_text()
    ###################################################################
    def reset_framenumber_text( self ):
        """Set text labels for image windows."""
        for wnd in self.img_winds:
            if self.print_n_obs:
                text = "frame %d: %d flies"%(wnd['framenumber'], wnd['n_obs'])
            else:
                text = "frame %d"%wnd['framenumber']
            wnd['stext'].SetLabel( text )

        if self.print_n_obs:
            self.OnResize( None )
            # should also resize if n_obs used to be true and is now false


    ###################################################################
    # OnResetFrames()
    ###################################################################
    def OnResetFrames( self, evt ):
        """GUI callback to set frame numbers."""
        self.reset_frame_numbers()
        self.redraw()


    ###################################################################
    # OnPrevButton()
    ###################################################################
    def OnPrevButton( self, evt ):
        """'Previous' button pressed. Go back to last step."""
        if len( self.prev_calls ) == 0: return # should be disabled, actually
        
        prev_call = self.prev_calls.pop()
        if len( self.prev_calls ) == 0:
            self.prev_button.Enable( False )
        self.next_button.SetLabel( self.default_next_label )
        prev_call()
        

    ###################################################################
    # OnNextButton()
    ###################################################################
    def OnNextButton( self, evt ):
        """'Next' button pressed. Execute some calls."""
        if len( self.next_calls ) == 0:
            self.frame.Close()
        else:
            if self.cur_call is not None:
                self.prev_calls.append( self.cur_call )
            self.prev_button.Enable( True )
            for call in self.next_calls:
                call()


    ###################################################################
    # remember_button_states()
    ###################################################################
    def remember_button_states( self ):
        """Remember enablement states of next_button and prev_button."""
        if self.__button_reset_timer_running: return
        self.__prev_enabled = self.prev_button.Enabled
        self.__next_enabled = self.next_button.Enabled


    ###################################################################
    # reset_button_states()
    ###################################################################
    def reset_button_states( self ):
        """Reset enablement states of buttons to remembered values."""
        # This is a workaround for a complicated Windows-specific issue
        # with dialog boxes in Wx 2.9.4
        self.prev_button.Enable( self.__prev_enabled )
        self.next_button.Enable( self.__next_enabled )

        if self.next_button.Enabled != self.__next_enabled:
            self.__button_reset_timer_running = True
            Timer( 0.1, self.call_reset_button_states ).start()
        else:
            self.__button_reset_timer_running = False

    def call_reset_button_states( self ):
        """Call reset_button_states on the main thread."""
        wx.CallAfter( self.reset_button_states )


    ###################################################################
    # init_no_bg()
    ###################################################################
    def init_no_bg( self ):
        """Set instructions for initializing background."""
        self.showing_image_type = params.SHOW_RAW_FRAME
        self.show_multi()
        self.cleanup_controls()

        self.title_text.SetLabel( "Initialize background" )
        self.instruction_text.SetLabel( "Ctrax depends on an unchanging background, which is subtracted away from each frame of video to leave only the moving objects (i.e., the flies). First, are your flies dark on a light background or light on a dark background?" )
        self.frame.Layout()

        self.new_control_radio_box( ["dark on light background", "light on dark background", "not so clean"] )
        
        self.cur_call = self.init_no_bg
        self.next_calls = [self.set_bg_type]


    ###################################################################
    # set_bg_type()
    ###################################################################
    def set_bg_type( self ):
        """Use current value of radio control to determine background type."""
        selection = self.get_radio_control_selection()
        if selection == 0:
            self.bg.bg_type = 'dark_on_light'
        elif selection == 1:
            self.bg.bg_type = 'light_on_dark'
        else:
            self.bg.bg_type = 'other'

        self.bg.use_median = True # best default?

        # calculate background
        self.remember_button_states()
        success = self.recalculate_bg_callback()
        self.reset_button_states()

        if success:
            self.redraw()
            self.choose_roi_type()
        else:
            self.OnPrevButton( None )


    ###################################################################
    # choose_roi_type()
    ###################################################################
    def choose_roi_type( self ):
        """Allow user to select region-of-interest type."""
        self.showing_image_type = params.SHOW_RAW_FRAME
        self.show_multi()
        self.cleanup_controls()

        self.title_text.SetLabel( "Define region of interest" )
        self.instruction_text.SetLabel( "We can define a region of interest to prevent tracking in areas where no flies will ever be. What shape is your arena or tracking area?" )

        self.new_control_radio_box( ["circular", "polygonal", "don't use region of interest"] )

        self.frame.Layout()
        
        self.cur_call = self.choose_roi_type
        self.next_calls = [self.set_roi_type]


    ###################################################################
    # set_roi_type()
    ###################################################################
    def set_roi_type( self ):
        """Use current value of radio control to set region-of-interest type."""
        params.do_set_circular_arena = False
        self.prev_button.Enable( False )
        self.next_button.Enable( False )
        
        selection = self.get_radio_control_selection()
        if selection == 0:
            # circular
            params.do_set_circular_arena = True
            self.roi = SetArena( self.frame, self.bg.center )
            self.roi.frame.Bind( wx.EVT_CLOSE, self.OnQuitCircularArena )
            
        elif selection == 1:
            # polygonal
            self.roi = ROI( self.frame, self.bg )
            self.roi.frame.Bind( wx.EVT_BUTTON, self.OnQuitPolygonalArena, id=xrc.XRCID("quit_button") )
            self.roi.frame.Bind( wx.EVT_CLOSE, self.OnQuitPolygonalArena )
            
        else:
            # none
            self.bg.SetROI( num.ones( self.img_size, num.bool ) )
            params.roipolygons = []
            self.bg.UpdateIsArena()

            self.prev_button.Enable( True )
            self.next_button.Enable( True )

            self.check_bg()

        if hasattr( self, 'roi' ):
            self.roi.frame.Show()


    ###################################################################
    # OnQuitCircularArena()
    ###################################################################
    def OnQuitCircularArena( self, evt ):
        """Circular arena ROI was set. Grab parameters and continue."""
        [params.arena_center_x, params.arena_center_y,
         params.arena_radius] = self.roi.GetArenaParameters()
        self.bg.UpdateIsArena()

        self.roi.frame.Destroy()
        delattr( self, 'roi' )

        self.prev_button.Enable( True )
        self.next_button.Enable( True )

        self.check_bg()


    ###################################################################
    # OnQuitPolygonalArena()
    ###################################################################
    def OnQuitPolygonalArena( self, evt ):
        """Polygonal arena ROI was set. Grab parameters and continue."""
        if not self.roi.CheckSave():
            # user pressed "cancel"
            return

        if 'darwin' in sys.platform:
            # should Destroy() as in OnQuitCircular, but it crashes on Mac
            self.roi.frame.Hide()
        else:
            self.roi.frame.Destroy()
        delattr( self, 'roi' )

        self.prev_button.Enable( True )
        self.next_button.Enable( True )

        self.check_bg()


    ###################################################################
    # check_bg()
    ###################################################################
    def check_bg( self ):
        """Prompt user to check over and correct background image."""
        if self.isSBFMF:
            # skip this step
            self.choose_high_threshold()
            return
        
        self.showing_image_type = params.SHOW_BACKGROUND
        self.show_single()
        self.cleanup_controls()

        self.title_text.SetLabel( "Check background" )
        self.instruction_text.SetLabel( "Look carefully at the background image. It should look exactly like an empty arena. If there are any \"ghosts\" of flies in the background image, then flies entering those areas will likely not be tracked correctly. Does the image have \"ghost flies\"?" )

        self.new_control_radio_box( ["yes, fix ghosts", "no, it's fine"] )

        self.frame.Layout()

        if self.prev_calls[-1] == self.check_bg:
            self.cur_call = None
        else:
            self.cur_call = self.check_bg
        self.next_calls = [self.decide_fix_bg]


    ###################################################################
    # decide_fix_bg()
    ###################################################################
    def decide_fix_bg( self ):
        """Use current value of radio control to determine if background needs to be fixed."""
        self.prev_button.Enable( False )
        self.next_button.Enable( False )
        selection = self.get_radio_control_selection()
        if selection == 0:
            self.roi = FixBG( self.frame, self.bg )
            self.roi.frame.Bind( wx.EVT_BUTTON, self.OnQuitFixBg, id=xrc.XRCID("quit_button") )
            self.roi.frame.Bind( wx.EVT_CLOSE, self.OnQuitFixBg )
            self.roi.frame.Show()

        else:
            self.prev_button.Enable( True )
            self.next_button.Enable( True )
            self.choose_high_threshold()


    ###################################################################
    # OnQuitFixBg()
    ###################################################################
    def OnQuitFixBg( self, evt ):
        """Fix-background dialog was closed. Continue."""
        self.OnQuitPolygonalArena( evt )
        self.choose_high_threshold()


    ###################################################################
    # choose_high_threshold()
    ###################################################################
    def choose_high_threshold( self ):
        """Allow user to set high background threshold."""
        self.showing_image_type = params.SHOW_THRESH
        self.show_multi()
        self.cleanup_controls()

        self.print_n_obs = True
        self.reset_framenumber_text()
        
        self.title_text.SetLabel( "Choose high threshold" )
        self.instruction_text.SetLabel( "After subtracting the background, Ctrax uses thresholds to determine which \"foreground\" pixels truly belong to flies. The high threshold is the minumum deviation from background required in at least one pixel of each fly. Set it as high as you can while each fly is visible." )

        self.new_slider( params.n_bg_std_thresh, "high threshold: %3d"%params.n_bg_std_thresh )
        self.slider.Bind( wx.EVT_SCROLL, self.OnHighThreshChanged )
        self.frame.Layout()

        self.cur_call = self.choose_high_threshold
        self.next_calls = [self.choose_low_threshold]


    ###################################################################
    # OnHighThreshChanged()
    ###################################################################
    def OnHighThreshChanged( self, evt ):
        """Value changed in high threshold slider. Update values in bg."""
        evt.Skip()
        new_thresh = self.slider.GetThumbPosition()
        if new_thresh != params.n_bg_std_thresh:
            params.n_bg_std_thresh = new_thresh
            self.slider_text.SetLabel( "high threshold: %3d"%params.n_bg_std_thresh )
            self.redraw()


    ###################################################################
    # choose_low_threshold()
    ###################################################################
    def choose_low_threshold( self ):
        """Allow user to set low background threshold."""
        if params.n_bg_std_thresh_low > params.n_bg_std_thresh:
            params.n_bg_std_thresh_low = params.n_bg_std_thresh
        
        self.showing_image_type = params.SHOW_CC
        self.show_multi()
        self.cleanup_controls()

        self.print_n_obs = True
        self.reset_framenumber_text()

        self.title_text.SetLabel( "Choose low threshold" )
        self.instruction_text.SetLabel( "The high threshold is the minimum signal strength required in at least one pixel of each connected component (fly). The low threshold defines the edges of the flies - pixels are foreground if they are above the low threshold and also near a pixel above the high threshold. Set it to make each fly distinct." )

        self.new_slider( params.n_bg_std_thresh_low, "low threshold: %3d"%params.n_bg_std_thresh_low )
        self.slider.Bind( wx.EVT_SCROLL, self.OnLowThreshChanged )
        self.frame.Layout()

        self.cur_call = self.choose_low_threshold
        self.next_calls = [self.choose_shape_method]


    ###################################################################
    # OnLowThreshChanged()
    ###################################################################
    def OnLowThreshChanged( self, evt ):
        """Value changed in low threshold slider. Update values in bg."""
        evt.Skip()
        new_thresh = self.slider.GetThumbPosition()
        if new_thresh <= params.n_bg_std_thresh and \
               new_thresh != params.n_bg_std_thresh_low:
            params.n_bg_std_thresh_low = new_thresh
            self.slider_text.SetLabel( "low threshold: %3d"%params.n_bg_std_thresh_low )
            self.redraw()
        elif new_thresh > params.n_bg_std_thresh:
            self.slider.SetThumbPosition( params.n_bg_std_thresh - 1 )


    ###################################################################
    # choose_shape_method()
    ###################################################################
    def choose_shape_method( self ):
        """Choose whether to automatically compute fly shape."""
        self.cleanup_controls()
        
        self.title_text.SetLabel( "Auto-compute shape?" )
        self.instruction_text.SetLabel( "Setting bounds on the shapes of real flies in the movie helps allow Ctrax to discard spurious detections. These bounds can be estimated automatically. However, if your current threshold settings have many spurious detections, the automatic computation might prefer false flies to the real ones." )

        self.new_control_radio_box( ["auto-compute shapes", "manually set bounds"] )
        self.frame.Layout()

        self.cur_call = self.choose_shape_method
        self.next_calls = [self.set_shape_method]

    
    ###################################################################
    # set_shape_method()
    ###################################################################
    def set_shape_method( self ):
        """A shape-computation method was selected. Take action."""
        selection = self.get_radio_control_selection()
        if selection == 0:
            # auto-compute shape
            wx.BeginBusyCursor()
            wx.Yield()
            self.remember_button_states()
            success = est_shape( self.bg, self.frame )
            self.reset_button_states()
            wx.EndBusyCursor()

            if success:
                self.show_shape_results()
            else:
                self.OnPrevButton( None )
            
        elif selection == 1:
            # launch settings for manual shape setting
            self.prev_button.Enable( False )
            self.next_button.Enable( False )
            
            self.tracking_settings = TrackingSettings( self.frame, self.bg, 0 )
            self.tracking_settings.frame.Bind( wx.EVT_CLOSE, self.OnQuitTrackingSettings )
            self.tracking_settings.frame.Bind( wx.EVT_BUTTON, self.OnQuitTrackingSettings, id=xrc.XRCID("done") )
            self.tracking_settings.frame.Show()

            # hack to switch dialog to manual settings
            self.tracking_settings.automatic_shape_input.SetValue( False )
            self.tracking_settings.manual_shape_input.SetValue( True )
            self.tracking_settings.OnAutomatic( None )

 
    ###################################################################
    # OnQuitTrackingSettings()
    ###################################################################
    def OnQuitTrackingSettings( self, evt ):
        """Tracking settings dialog was closed. Continue toward tracking."""
        self.tracking_settings.frame.Destroy()
        delattr( self, 'tracking_settings' )

        self.prev_button.Enable( True )
        self.next_button.Enable( True )

        self.show_shape_results()


    ###################################################################
    # show_shape_results()
    ###################################################################
    def show_shape_results( self ):
        """Shapes were computed. Show results."""
        self.showing_image_type = params.SHOW_ELLIPSES
        self.show_multi()
        self.cleanup_controls()

        self.print_n_obs = True
        self.reset_framenumber_text()

        self.title_text.SetLabel( "Start tracking?" )
        self.instruction_text.SetLabel( "Check whether each fly is surrounded by exactly one ellipse in each frame. (You can drag in the frame to zoom in; double-click to zoom out. Also press \"Randomize Frames\" a few times.) Close this window to finish without tracking, or press \"Track\"!" )
        self.frame.Layout()

        self.next_calls = [self.start_tracking]
        self.next_button.SetLabel( "Track" )

    
    ###################################################################
    # start_tracking()
    ###################################################################
    def start_tracking( self ):
        """Done with wizard. Start tracking."""
        if 'darwin' in sys.platform:
            # Close() crashes on Mac, after tracking finishes
            self.frame.Hide()
        else:
            self.frame.Close()
        self.starttracking_callback()

        
    ###################################################################
    # test_call()
    ###################################################################
    def test_call( self ):
        self.title_text.SetLabel( "Test complete" )
        self.instruction_text.SetLabel( "You've reached the test callback. This is only a test." )
        self.frame.Layout()

        self.next_calls = []
        self.next_button.SetLabel( "Finish" )
        



from scipy.misc.pilutil import imresize
import numpy as num
import wx
import string

def img_normalize(img):

    img -= num.min(img)
    v = num.max(img)
    if v > 0:
        img /= v

    return img

def double2mono8(img_d,donormalize=True):

    img_8 = img_d.copy()
    if not img_8.dtype == num.dtype('double'):
        img_8 = img_8.astype(num.double)
    if donormalize:
        img_8 -= num.min(img_8)
        v = num.max(img_8)
        if v > 0:
            img_8 *= 255./v
    img_8 = img_8.round().astype(num.uint8)

    return img_8

def draw_image(img,windowsize=None,zoomaxes=None):

    # get axes if not input
    if zoomaxes is None:
        zoomaxes = [0,img.shape[1]-1,0,img.shape[0]-1]
    # check to make sure valid
    if (int(zoomaxes[3]) < int(zoomaxes[2])) or (int(zoomaxes[1]) < int(zoomaxes[0])):
        raise ValueError('Invalid zoom axes input')
    # crop image
    scale_img = img.copy()
    scale_img = scale_img[int(zoomaxes[2]):int(zoomaxes[3]+1),int(zoomaxes[0]):int(zoomaxes[1]+1)]
    
    # resize image so that it fits in the window
    if windowsize is not None:
        scale_x = float(windowsize[1])/float(scale_img.shape[1])
        scale_y = float(windowsize[0])/float(scale_img.shape[0])
        if scale_x < scale_y:
            size_x = windowsize[1]
            size_y = scale_img.shape[0]*scale_x
            resize = scale_x
        else:
            size_y = windowsize[0]
            size_x = scale_img.shape[1]*scale_y
            resize = scale_y
        scale_img = imresize(scale_img,(int(size_y),int(size_x)))

    # create an rgb image out of img
    if scale_img.ndim == 3:
        if not (scale_img.dtype == 'uint8'):
            scale_img = scale_img.astype('uint8')
    elif scale_img.dtype == 'uint8':
        scale_img = to_rgb8('MONO8',scale_img)
    elif scale_img.dtype == 'double':
        # scale to between 0 and 255
        scale_img = num.astype(img_normalize(scale_img)*255.,'uint8')
        scale_img = to_rgb8('MONO8',scale_img)

    #print 'image shape:'
    #print scale_img.shape
    #print 'resize: %f'%resize
    
    # create a bitmap out of image
    h,w,three = scale_img.shape
    img_size = [h,w]
    image = wx.EmptyImage(w,h)
    image.SetData( scale_img.tostring() )
    bmp = wx.BitmapFromImage(image)

    return (bmp,resize,img_size)

def draw_annotated_image(img,pointlists=None,linelists=None,circlelists=None,
                         windowsize=None,zoomaxes=None,
                         pointcolors=None,linecolors=None,circlecolors=None,
                         pointsizes=None,linewidth=None,circlewidths=None):

    #print 'in draw_annotated_image'

    # get axes if not input
    if zoomaxes is None:
        zoomaxes = [0,img.shape[1]-1,0,img.shape[0]-1]
    # check to make sure valid
    if (int(zoomaxes[3]) < int(zoomaxes[2])) or (int(zoomaxes[1]) < int(zoomaxes[0])):
        raise ValueError('Invalid zoom axes input')
    # crop image
    scale_img = img.copy()
    scale_img = scale_img[int(zoomaxes[2]):int(zoomaxes[3]+1),int(zoomaxes[0]):int(zoomaxes[1]+1)]
    xoffset = -zoomaxes[0]
    yoffset = -zoomaxes[2]

    #print 'zoomaxes = ' + str(zoomaxes)

    # resize image so that it fits in the window
    if windowsize is not None:
        scale_x = float(windowsize[1])/float(scale_img.shape[1])
        scale_y = float(windowsize[0])/float(scale_img.shape[0])
        if scale_x < scale_y:
            size_x = windowsize[1]
            size_y = scale_img.shape[0]*scale_x
            resize = scale_x
        else:
            size_y = windowsize[0]
            size_x = scale_img.shape[1]*scale_y
            resize = scale_y
        scale_img = imresize(scale_img,(int(size_y),int(size_x)))

    #print 'current size of scale_img = ' + str(scale_img.shape)

    # create an rgb image out of img
    if scale_img.ndim == 3:
        if not (scale_img.dtype == 'uint8'):
            scale_img = scale_img.astype('uint8')
    elif scale_img.dtype == 'uint8':
        scale_img = to_rgb8('MONO8',scale_img)
    elif scale_img.dtype == 'double':
        # scale to between 0 and 255
        scale_img = num.astype(img_normalize(scale_img)*255.,'uint8')
        scale_img = to_rgb8('MONO8',scale_img)

    #print 'image shape after converting to rgb:'
    #print scale_img.shape
    #print 'resize: %f'%resize
    
    # create a bitmap out of image
    h,w,three = scale_img.shape
    img_size = [h,w]
    image = wx.EmptyImage(w,h)

    #print 'created empty image of size (%d,%d)'%(w,h)
    
    image.SetData( scale_img.tostring() )

    #print 'set the data'
    
    bmp = wx.BitmapFromImage(image)

    #print 'created bmp'

    # draw into bmp
    drawDC = wx.MemoryDC()
    drawDC.SelectObject( bmp ) # draw into bmp
    # set default point color
    drawDC.SetPen(wx.Pen('GREEN'))
    drawDC.SetBrush(wx.Brush(wx.Colour(255,255,255), wx.TRANSPARENT))
    # by default set point radius to 8
    point_radius=8

    #print 'starting to draw stuff'

    if pointlists is not None:
        pointcolor = 'GREEN'
        for i,points in enumerate(pointlists):
            # set color
            if (pointcolors is not None) and (len(pointcolors) > i):
                pointcolor = wx.Colour(pointcolors[i][0],pointcolors[i][1],pointcolors[i][2])
            if (pointsizes is not None) and (len(pointsizes) > i):
                point_radius = pointsizes[i]
            drawDC.SetPen(wx.Pen(colour=pointcolor,width=point_radius))
            # set radius
            for j,pt in enumerate(points):
                # draw a circle
                x = int((xoffset+pt[0])*resize)
                y = int((yoffset+pt[1])*resize)
                if (x >= 0) and (x < img_size[1]) and \
                   (y >= 0) and (y < img_size[0]):
                    drawDC.DrawCircle(x,y,point_radius)

    #print 'finished drawing points'

    if linelists is not None:
        # set default line color
        linecolor = 'GREEN'
        # set default line width
        if linewidth is None:
            linewidth = 1

        for i,lines in enumerate(linelists):

            #print i

            # create a list of wxPoints
            points = []
            for j,pt in enumerate(lines):
                x = int((xoffset+pt[0])*resize)
                y = int((yoffset+pt[1])*resize)
                newpoint = wx.Point(x,y)
                if (j < 1) or not (newpoint == lastpoint):
                    points.append(newpoint)
                    lastpoint = newpoint

            if len(points) == 0:
                continue
            if len(points) == 1:
                points.append(newpoint)

            # set color
            if (linecolors is not None) and (len(linecolors) > i):
                linecolor = wx.Colour(linecolors[i][0],linecolors[i][1],
                                      linecolors[i][2])

            drawDC.SetPen(wx.Pen(colour=linecolor,width=linewidth))

            #print 'drawing line with color'
            #print linecolor
            #print 'width'
            #print linewidth
            #print 'points'
            #print points

            # draw the lines
            drawDC.DrawLines(points)

    #print 'finished drawing lines'

    if circlelists is not None:
        circlecolor = 'GREEN'
        for i,circles in enumerate(circlelists):
            # set color
            if (circlecolors is not None) and (len(circlecolors) > i):
                circlecolor = wx.Colour(circlecolors[i][0],circlecolors[i][1],circlecolors[i][2])
            if (circlewidths is not None) and (len(circlewidths) > i):
                circlewidth = circlewidths[i]
            drawDC.SetPen(wx.Pen(colour=circlecolor,width=circlewidth))
            # set radius
            if (circlewidths is not None) and (len(circlewidths) > i):
                circlewidth = circlewidths[i]
            for j,circle in enumerate(circles):
                # draw a circle
                x = int((xoffset+circle[0])*resize)
                y = int((yoffset+circle[1])*resize)
                r = int(circle[2]*resize)
                if (x >= 0) and (x < img_size[1]) and \
                   (y >= 0) and (y < img_size[0]):
                    drawDC.DrawCircle(x,y,r)

    #print 'finished drawing circles'
                    
    #print 'leaving draw_annotated_image'

    return (bmp,resize,img_size)

def add_annotations(bmp,resize,pointlists=None,linelists=None,
                    zoomaxes=[0,10,0,10],
                    pointcolors=None,linecolors=None,
                    pointsizes=None,linewidth=None):

    # check to make sure valid
    if (zoomaxes is None) or (int(zoomaxes[3]) < int(zoomaxes[2])) \
           or (int(zoomaxes[1]) < int(zoomaxes[0])):
        raise ValueError('Invalid zoom axes input')
    xoffset = -zoomaxes[0]
    yoffset = -zoomaxes[2]

    # draw into bmp
    drawDC = wx.MemoryDC()
    drawDC.SelectObject( bmp ) # draw into bmp
    # set default point color
    drawDC.SetPen(wx.Pen('GREEN'))
    drawDC.SetBrush(wx.Brush(wx.Colour(255,255,255), wx.TRANSPARENT))
    # by default set point radius to 8
    point_radius=8

    if pointlists is not None:
        for i,points in enumerate(pointlists):
            # set color
            if (pointcolors is not None) and (len(pointcolors) > i):
                c = wx.Colour(pointcolors[i][0],pointcolors[i][1],pointcolors[i][2])
                drawDC.SetPen(wx.Pen(c))
            # set radius
            if (pointsizes is not None) and (len(pointsizes) > i):
                point_radius = pointsizes[i]
            for j,pt in enumerate(points):
                # draw a circle
                x = int((xoffset+pt[0])*resize)
                y = int((yoffset+pt[1])*resize)
                drawDC.DrawCircle(x,y,point_radius)

    if linelists is not None:
        # set default line color
        linecolor = 'GREEN'
        # set default line width
        if linewidth is None:
            linewidth = 1

        for i,lines in enumerate(linelists):

            #print i

            # create a list of wxPoints
            points = []
            for j,pt in enumerate(lines):
                x = int((xoffset+pt[0])*resize)
                y = int((yoffset+pt[1])*resize)
                newpoint = wx.Point(x,y)
                if (j < 1) or not (newpoint == lastpoint):
                    points.append(newpoint)
                    lastpoint = newpoint

            if len(points) == 0:
                continue
            if len(points) == 1:
                points.append(newpoint)

            # set color
            if (linecolors is not None) and (len(linecolors) > i):
                linecolor = wx.Colour(linecolors[i][0],linecolors[i][1],
                                      linecolors[i][2])
                
            drawDC.SetPen(wx.Pen(colour=linecolor,width=linewidth))

            #print 'drawing line with color'
            #print linecolor
            #print 'width'
            #print linewidth
            #print 'points'
            #print points

            # draw the lines
            drawDC.DrawLines(points)

    return bmp

def draw_circle(x,y,r,color=(0,255,0),step=10*num.pi/180.):

    lines = []

    # first point
    aa = 0
    xx = x + num.cos(aa)*r
    yy = y + num.sin(aa)*r
    
    for aa in num.arange( 0., (2.*num.pi+step), step ):
        xprev = xx
        yprev = yy
        xx = x + num.cos(aa)*r
        yy = y + num.sin(aa)*r
        lines.append([xprev+1,yprev+1,xx+1,yy+1,color])

    return lines

def draw_line(x0,y0,x1,y1,color=(0,255,0)):

    linesegs = [[x0,y0,x1,y1,color]]

    return linesegs

def draw_arc(xc,yc,r,theta0,theta1,color=(0,255,0),step=10*num.pi/180.):

    lines = []

    # first point
    aa = theta0
    xx = xc + num.cos(aa)*r
    yy = yc + num.sin(aa)*r
    
    for aa in num.arange( theta0, theta1, step ):
        xprev = xx
        yprev = yy
        xx = xc + num.cos(aa)*r
        yy = yc + num.sin(aa)*r
        lines.append([xprev+1,yprev+1,xx+1,yy+1,color])

    return lines

def zoom_linesegs_and_image(linesegs,im,zoomaxes):

    if (zoomaxes is None) or (int(zoomaxes[3]) < int(zoomaxes[2])) \
           or (int(zoomaxes[1]) < int(zoomaxes[0])):
        raise ValueError('Invalid zoom axes input')

    # extract the relevant part of the image
    zoomim = im[int(zoomaxes[2]):int(zoomaxes[3]+1),int(zoomaxes[0]):int(zoomaxes[1]+1)]

    # move the line segments relative to the zoom box
    xoffset = -int(zoomaxes[0])-.5
    yoffset = -int(zoomaxes[2])-.5

    for (i,line) in enumerate(linesegs):
        linesegs[i][0] += xoffset
        linesegs[i][1] += yoffset
        linesegs[i][2] += xoffset
        linesegs[i][3] += yoffset

    return (linesegs,zoomim)

def separate_linesegs_colors(linesegs):

    lines = []
    colors = []
    for (i,line) in enumerate(linesegs):
        lines.append(line[0:4])
        color = (line[4][0]/255.,line[4][1]/255.,line[4][2]/255.,1)
        colors.append(color)

    return (lines,colors)

def to_rgb8(inputtype,img):
    
    if string.lower(inputtype) == 'mono8':
        rgb_img = img.reshape(img.shape[0],img.shape[1],1)
        rgb_img = num.tile(rgb_img,(1,1,3))
    else:
        print 'Inputtype: ' + inputtype + ' not supported.'
        rgb_img = img

    return rgb_img



#!/usr/bin/env python

# Copyright (c) 2009, Giampaolo Rodola'. All rights reserved.
# Use of this source code is governed by a BSD-style license that can be
# found in the LICENSE file.

"""This module is deprecated as exceptions are defined in _error.py
and are supposed to be accessed from 'psutil' namespace as in:
- psutil.NoSuchProcess
- psutil.AccessDenied
- psutil.TimeoutExpired
"""

import warnings
from psutil._error import *

warnings.warn("psutil.error module is deprecated and scheduled for removal; " \
              "use psutil namespace instead", category=DeprecationWarning,
               stacklevel=2)



import numpy as num
import scipy.cluster.vq as vq
#import scipy.linalg.decomp as decomp
from version import DEBUG

d = 2

def clusterdistfun(x,c):
    n = x.shape[0]
    nclusts = c.shape[0]
    D = num.empty((nclusts,n))
    for i in range(nclusts):
        D[i,:] = num.sum((x - c[i,:])**2,axis=1)
    return D

def furthestfirst(x,k,mu0=None,start='mean'):
    # number of data points
    n = x.shape[0]
    # data dimensionality: always 2
    # d = x.shape[1]
    
    # returned centers
    mu = num.empty((k,d))
    
    # initialize first center
    if num.any(mu0) == False:
        if start == 'mean':
            # choose the first mean to be the mean of all the data
            mu[0,:] = num.mean(x,axis=0)
        else:
            # choose a random point to be the first center
            mu[0,:] = x[num.random.randint(0,x.shape[0],1)[0],:]
    else:
        mu[0,:] = mu0
    
    # initialize distance to all centers to be infinity
    Dall = num.empty((k,n))
    Dall[:] = num.inf

    for i in range(1,k):
        # compute distance to all centers from all points
        Dall[i-1,:] = num.sum((x - mu[i-1,:])**2,axis=1)
        # compute the minimum distance from all points to the centers
        D = num.amin(Dall,axis=0)
        # choose the point furthest from all centers as the next center
        j = num.argmax(D)
        mu[i,:] = x[j,:]

    Dall[k-1,:] = num.sum((x - mu[k-1,:])**2,axis=1)
    idx = num.argmin(Dall,axis=0)

    return (mu,idx)

def gmminit(x,k,weights=None,kmeansiter=20,kmeansthresh=.001):

    n = x.shape[0]
    # 2d
    #d = x.shape[1]

    # initialize using furthest-first clustering
    (mu,idx) = furthestfirst(x,k,start='random')

    # use k-means, beginning from furthest-first
    (mu,dmin) = vq.kmeans(x,mu,kmeansiter,kmeansthresh)

    # get labels for each data point
    D = clusterdistfun(x,mu)
    idx = num.argmin(D,axis=0)

    # allocate covariance and priors
    S = num.empty((d,d,k))
    priors = num.empty(k)

    if num.any(weights) == False:
        # unweighted
        for i in range(k):
            # compute prior for each cluster
            nidx = num.sum(num.double(idx==i))
            priors[i] = nidx
            # compute mean for each cluster
            mu[i,:] = num.mean(x[idx==i,:],axis=0)
            # compute covariance for each cluster
            diffs = x[idx==i,:] - mu[i,:].reshape(1,d)
            S[:,:,i] = num.dot(num.transpose(diffs),diffs) / priors[i]
    else:
        # replicate weights
        weights = weights.reshape(n,1)
        for i in range(k):
            # compute prior for each cluster
            nidx = num.sum(num.double(idx==i))
            priors[i] = num.sum(weights[idx==i])
            # compute mean for each cluster
            mu[i,:] = num.sum(weights[idx==i]*x[idx==i,:],axis=0)/priors[i]
            # compute covariance for each cluster
            diffs = x[idx==i,:] - mu[i,:].reshape(1,d)
            diffs *= num.sqrt(weights[idx==i])
            S[:,:,i] = num.dot(num.transpose(diffs),diffs) / priors[i]

    # normalize priors
    priors = priors / num.sum(priors)

    # return
    return (mu,S,priors)

def gmm(x,k,weights=None,nreplicates=10,kmeansiter=20,kmeansthresh=.001,emiters=100,emthresh=.001,mincov=.01):

    # number of data points
    n = x.shape[0]
    # dimensionality of each data point: 2d
    #d = x.shape[1]
    # initialize min error
    minerr = num.inf

    # replicate many times
    for rep in range(nreplicates):

        # initialize randomly
        (mu,S,priors) = gmminit(x,k,weights,kmeansiter,kmeansthresh)

        # optimize fit using EM
        (mu,S,priors,gamma,err) = gmmem(x,mu,S,priors,weights,emiters,emthresh,mincov)

        if rep == 0 or err < minerr:
            mubest = mu
            Sbest = S
            priorsbest = priors
            minerr = err
            gammabest = gamma

    return (mubest,Sbest,priorsbest,gammabest,minerr)

def gmmmemberships(mu,S,priors,x,weights=1,initcovars=None):
    if initcovars is None:
        initcovars = S.copy()

    # number of data points
    n = x.shape[0]
    # dimensionality of data: 2d
    #d = x.shape[1]
    # number of clusters
    k = mu.shape[0]

    # allocate output
    gamma = num.empty((n,k))

    # normalization constant
    normal = 2*num.pi
    #normal = (2.0*num.pi)**(num.double(d)/2.0)
    #print 'd = %d, normal = %f' % (d,normal)
    #print 'mu = '
    #print mu
    #print 'S = '
    #for j in range(k):
    #    print S[:,:,j]
    #print 'priors = '
    #print priors
    #print 'weights = '
    #print weights

    # compute the activation for each data point
    for j in range(k):
        #print 'j = %d' % j
        #print 'mu[%d,:] = '%j + str(mu[j,:])
        #print 'S[:,:,%d] = '%j + str(S[:,:,j])
        diffs = x - mu[j,:]
        #print 'diffs = '
        #print diffs

        zz = S[0,0,j]*S[1,1,j] - S[0,1,j]**2
        if zz <= 0:
            if DEBUG: print 'S[:,:,%d] = '%j + str(S[:,:,j]) + ' is singular'
            if DEBUG: print 'Reverting to initcovars[:,:,%d] = '%j + str(initcovars[:,:,j])
            S[:,:,j] = initcovars[:,:,j]
            zz = S[0,0,j]*S[1,1,j] - S[0,1,j]**2

        temp = (diffs[:,0]**2*S[1,1,j]
                - 2*diffs[:,0]*diffs[:,1]*S[0,1,j]
                + diffs[:,1]**2*S[0,0,j]) / zz

        gamma[:,j] = num.exp(-.5*temp)/(normal*num.sqrt(zz))
        #gamma[:,j] = num.exp(-.5*temp)/(normal*num.prod(num.diag(c)))
        #print 'gamma[:,%d] = ' % j
        #print gamma[:,j]

    # include prior
    gamma *= priors
    #print 'after including prior, gamma = '
    #print gamma

    # compute negative log likelihood
    e = -num.sum(num.log(num.sum(gamma,axis=1))*weights)
    
    s = num.sum(gamma,axis=1)
    #print 's = '
    #print s
    # make sure we don't divide by 0
    s[s==0] = 1
    gamma /= s.reshape(n,1)
    #print 'gamma = '
    #print gamma
    
    return (gamma,e)    

def gmmupdate(mu,S,priors,gamma,x,weights=1,mincov=.01,initcovars=None):

    if num.any(initcovars) == False:
        initcovars = S
    
    # number of data points
    n = gamma.shape[0]
    # number of clusters
    k = gamma.shape[1]
    # dimensionality of data: 2d
    #d = x.shape[1]

    gamma *= weights.reshape(n,1)

    # update the priors (note that it has not been normalized yet)
    priors = num.sum(gamma,axis=0)
    Z = priors.copy()
    sumpriors = num.sum(priors)
    if sumpriors > 0:
        priors /= sumpriors
        issmall = priors < .01
        issmall = issmall.any()
    else:
        if DEBUG: print "All priors are too small, reinitializing"
        issmall = True # num.bool_ NOT bool
    #print 'updated priors to ' + str(priors)

    # if any prior is to small, then reinitialize that cluster
    if issmall:
        fixsmallpriors(x,mu,S,priors,initcovars,gamma)
        priors = num.sum(gamma,axis=0)
        Z = priors.copy()
        priors /= num.sum(priors)
        if DEBUG: print 'after fixsmallpriors, priors is ' + str(priors)

    #issmall = issmall.any()
    if issmall:
        if DEBUG: 
            print 'outside fixsmallpriors'
            print 'reset mu = ' + str(mu)
            for i in range(k):
                print 'reset S[:,:,%d] = '%i + str(S[:,:,i])
            print 'reset priors = ' + str(priors)
        #print 'reset gamma = '
        #print gamma

    for i in range(k):
        # update the means
        mu[i,:] = num.sum(gamma[:,i].reshape(n,1)*x,axis=0)/Z[i]
        if DEBUG:
            if issmall: 
                print 'updated mu[%d,:] to '%i + str(mu[i,:])
        # update the covariances
        diffs = x - mu[i,:]
        #if issmall: print 'diffs = ' + str(diffs)
        diffs *= num.sqrt(gamma[:,i].reshape(n,1))
        #if issmall: print 'weighted diffs = ' + str(diffs)
        S[:,:,i] = (num.dot(num.transpose(diffs),diffs)) / Z[i]
        if DEBUG:
            if issmall: 
                print 'updated S[:,:,%d] to [%.4f,%.4f;%.4f,%.4f]'%(i,S[0,0,i],S[0,1,i],S[1,0,i],S[1,1,i])
        # make sure covariance is not too small
        if mincov > 0:
            # hard-coded 2x2
            eigval_T = S[0,0,i] + S[1,1,i]
            mineigval = eigval_T - num.sqrt(eigval_T**2/4 - S[1,1,i])
            if mineigval < mincov:
                S[:,:,i] = initcovars[:,:,i]
                if DEBUG: 
                    print 'mineigval = %.4f'%mineigval
                    print 'reinitializing covariance'
                    print 'initcovars[:,:,%d] = [%.4f,%.4f;%.4f,%.4f]'%(i,initcovars[0,0,i],initcovars[0,1,i],initcovars[1,0,i],initcovars[1,1,i])
                
def gmmem(x,mu0,S0,priors0,weights=None,niters=100,thresh=.001,mincov=.01):

    #print 'data = '
    #print x
    mu = mu0.copy()
    S = S0.copy()
    priors = priors0.copy()

    e = num.inf
    # store initial covariance in case covariance becomes too small
    if mincov > 0:
        for i in range(S.shape[2]):
            #print 'S initially is: '
            #print S[:,:,i]
            eigval_T = S[0,0,i] + S[1,1,i]
            mineigval = eigval_T - num.sqrt(eigval_T**2/4 - S[1,1,i])
            if num.isnan(mineigval) or mineigval < mincov:
                D, U = num.linalg.eig(S[:,:,i])
                if DEBUG: print "initial S[:,:,%d] is singular"%i
                if DEBUG: print "S[:,:,%d] = "%i
                if DEBUG: print str(S[:,:,i])
                D[D<mincov] = mincov
                S[:,:,i] = num.dot(num.dot(U,num.diag(D)),U.transpose())
                if DEBUG: print 'S[:,:,%d] reinitialized to: '%i
                if DEBUG: print S[:,:,i]
    initcovars = S.copy()

    for iter in range(niters):

        # E-step: compute memberships
        [gamma,newe] = gmmmemberships(mu,S,priors,x,weights,initcovars)

        # M-step: update parameters
        gmmupdate(mu,S,priors,gamma,x,weights,mincov,initcovars)

        # if we've converged, break
        if newe >= e - thresh:
            break
        
        e = newe

    [gamma,e] = gmmmemberships(mu,S,priors,x,weights,initcovars)

    return (mu,S,priors,gamma,e)

def fixsmallpriors(x,mu,S,priors,initcovars,gamma):

    #print 'calling fixsmallpriors with: '
    #print 'mu = ' + str(mu)
    #print 'S = '
    #for i in range(S.shape[2]):
    #    print S[:,:,i]
    #print 'priors = ' + str(priors)
    #for i in range(initcovars.shape[2]):
    #    print 'initcovars[:,:,%d]: '%i
    #    print initcovars[:,:,i]
    #    print 'initcovars[:,:,%d].shape: '%i + str(initcovars[:,:,i].shape)
        
    MINPRIOR = .01
    issmall = priors < .01
    if not issmall.any():
        return

    n = x.shape[0]
    # 2d
    #d = x.shape[1]
    k = mu.shape[0]

    # loop through all small priors
    smalli, = num.where(issmall)
    for i in smalli:

        if DEBUG: print 'fixing cluster %d with small prior = %f: '%(i,priors[i])

        # compute mixture density of each data point
        p = num.sum(gamma*priors,axis=1)

        #print 'samples: '
        #print x
        #print 'density of each sample: '
        #print p

        # choose the point with the smallest probability
        j = p.argmin()

        if DEBUG: print 'lowest density sample: x[%d] = '%j + str(x[j,:])

        # create a new cluster
        mu[i,:] = x[j,:]
        S[:,:,i] = initcovars[:,:,i]
        priors *= (1. - MINPRIOR)/(1.-priors[i])
        priors[i] = MINPRIOR

        if DEBUG: 
            print 'reset cluster %d to: '%i
            print 'mu = ' + str(mu[i,:])
            print 'S = '
            print S[:,:,i]
            print 'S.shape: ' + str(S[:,:,i].shape)
            print 'priors = ' + str(priors)

        # update gamma
        [gamma,newe] = gmmmemberships(mu,S,priors,x,1,initcovars)

#n0 = 100
#n1 = 100
#d = 3
#k = 2
#mu0 = num.array([0.,0.,0.])
#mu1 = num.array([5.,5.,5.])
#weights = num.hstack((10*num.ones(n0),num.ones(n1)))
#std0 = 1
#std1 = 1
#x0 = num.random.standard_normal(size=(n0,d))*std0 + mu0
#x1 = num.random.standard_normal(size=(n1,d))*std1 + mu1
#x = num.vstack((x0,x1))
#(mu,S,priors,err) = gmm(x,k,weights=weights)
#print 'centers = '
#print mu
#print 'covars = '
#print S[:,:,0]
#print S[:,:,1]
#print 'priors = '
#print priors
#print 'err = %f' % err
